@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,14 +29,10 @@
 
 #include "base/process.h"
 
-#include "base/const.h"
-
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
 #include <windows.h>
 #include <shellapi.h>
 #include <shlobj.h>
-#include "base/util.h"
-#include "third_party/mozc/sandbox/restricted_token_utils.h"
 #else
 #include <string.h>
 #include <sys/stat.h>
@@ -44,11 +40,9 @@
 #endif  // WINDOWS
 
 #ifdef OS_MACOSX
-#include <cstdlib>
 #include <fcntl.h>
 #include <signal.h>
 #include <spawn.h>  // for posix_spawn().
-#include <sys/stat.h>
 #include "base/mac_process.h"
 #endif  // OS_MACOSX
 
@@ -57,21 +51,36 @@
 #include <signal.h>
 #include <spawn.h>  // for posix_spawn().
 #include <sys/types.h>
-#include <unistd.h>
 #endif
 
+#include <cstdlib>
 #include <vector>
+
+#include "base/const.h"
+#include "base/file_util.h"
+#include "base/logging.h"
+#include "base/scoped_ptr.h"
+#include "base/system_util.h"
 #include "base/util.h"
 
-#ifndef OS_WINDOWS
+#ifdef OS_MACOSX
+// We do not use the global environ variable because it is unavailable
+// in Mac Framework/dynamic libraries.  Instead call _NSGetEnviron().
+// See the "PROGRAMMING" section of http://goo.gl/4Hq0D for the
+// detailed information.
+#include <crt_externs.h>
+static char **environ = *_NSGetEnviron();
+#elif !defined(OS_WIN)
 // Defined somewhere in libc. We can't pass NULL as the 6th argument of
 // posix_spawn() since Qt applications use (at least) DISPLAY and QT_IM_MODULE
 // environment variables.
 extern char **environ;
 #endif
 
-#ifdef OS_WINDOWS
+namespace mozc {
+
 namespace {
+#ifdef OS_WIN
 // ShellExecute to execute file in system dir.
 // Since Windows does not allow rename or delete a directory which
 // is set to the working directory by existing processes, we should
@@ -85,7 +94,7 @@ bool ShellExecuteInSystemDir(const wchar_t *verb,
                              INT show_command) {
   const int result =
       reinterpret_cast<int>(::ShellExecuteW(0, verb, file, parameters,
-                                            mozc::Util::GetSystemDir(),
+                                            SystemUtil::GetSystemDir(),
                                             show_command));
   LOG_IF(ERROR, result <= 32)
       << "ShellExecute failed."
@@ -95,90 +104,8 @@ bool ShellExecuteInSystemDir(const wchar_t *verb,
       << ", parameters: " << parameters;
   return result > 32;
 }
-
-bool BeginBackgroundProcessModeWin32(
-    mozc::ProsessPriorityClass *previous_mode) {
-  // GetCurrentProcess returns a pseudo handle, which is currently (HANDLE)-1.
-  // Unfortunately, (HANDLE)-1 is same to INVALID_HANDLE_VALUE so that we do
-  // not use mozc::ScopedHandle to avoid confusion.
-  const HANDLE current_process_handle = ::GetCurrentProcess();
-
-  // If |previous_mode| is NULL, set dummy address to it.
-  mozc::ProsessPriorityClass dummy;
-  if (previous_mode == NULL) {
-    previous_mode = &dummy;
-  }
-
-  bool succeeded = false;
-  *previous_mode = ::GetPriorityClass(current_process_handle);
-  if (*previous_mode == 0) {
-    LOG(ERROR) << "GetPriorityClass failed. error = " << GetLastError();
-    succeeded = false;
-  } else {
-    // Set process priority low
-    if (mozc::Util::IsVistaOrLater()) {
-      // Windows Vista introduced PROCESS_MODE_BACKGROUND_BEGIN, which can be
-      // specified only if |hProcess| parameter is a handle to the current
-      // process.
-      const DWORD error = ::SetPriorityClass(current_process_handle,
-                                             PROCESS_MODE_BACKGROUND_BEGIN);
-
-      succeeded = (error != 0 &&
-                   error != ERROR_PROCESS_MODE_ALREADY_BACKGROUND);
-      DLOG_IF(INFO, error == ERROR_PROCESS_MODE_ALREADY_BACKGROUND)
-          << "Already in background mode";
-      LOG_IF(ERROR, !succeeded)
-          << "SetPriorityClass failed. error = " << error;
-    } else {
-      const DWORD error = ::SetPriorityClass(current_process_handle,
-                                             BELOW_NORMAL_PRIORITY_CLASS);
-      succeeded = (error != 0);
-      LOG_IF(ERROR, !succeeded) << "SetPriorityClass failed. error = " << error;
-    }
-  }
-
-  // Actually CloseHandle with a pseudo handle has no effect.
-  ::CloseHandle(current_process_handle);
-
-  return succeeded;
-}
-
-bool EndBackgroundProcessModeWin32(
-    const mozc::ProsessPriorityClass *previous_mode) {
-  // GetCurrentProcess returns a pseudo handle, which is currently (HANDLE)-1.
-  // Unfortunately, (HANDLE)-1 is same to INVALID_HANDLE_VALUE so that we do
-  // not use mozc::ScopedHandle to avoid confusion.
-  const HANDLE current_process_handle = ::GetCurrentProcess();
-  bool succeeded = false;
-
-  // Set process priority to the previous one
-  if (mozc::Util::IsVistaOrLater()) {
-    const DWORD error = ::SetPriorityClass(current_process_handle,
-                                           PROCESS_MODE_BACKGROUND_END);
-    succeeded = (error != 0);
-    LOG_IF(ERROR, !succeeded) << "SetPriorityClass failed. error = " << error;
-  } else {
-    // Restore priority class
-    if (previous_mode == NULL) {
-      DLOG(INFO) << "previous_mode is NULL";
-      succeeded = false;
-    } else {
-      const DWORD error = ::SetPriorityClass(current_process_handle,
-                                             *previous_mode);
-      succeeded = (error != 0);
-      LOG_IF(ERROR, !succeeded) << "SetPriorityClass failed. error = " << error;
-    }
-  }
-
-  // Actually CloseHandle with a pseudo handle has no effect.
-  ::CloseHandle(current_process_handle);
-
-  return succeeded;
-}
-}  // anonymous namespace
-#endif  // OS_WINDOWS
-
-namespace mozc {
+#endif  // OS_WIN
+}  // namespace
 
 bool Process::OpenBrowser(const string &url) {
   // url must start with http:// or https:// or file://
@@ -188,7 +115,7 @@ bool Process::OpenBrowser(const string &url) {
     return false;
   }
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   wstring wurl;
   Util::UTF8ToWide(url.c_str(), &wurl);
   return ShellExecuteInSystemDir(L"open", wurl.c_str(), NULL, SW_SHOW);
@@ -209,7 +136,7 @@ bool Process::OpenBrowser(const string &url) {
 
 bool Process::SpawnProcess(const string &path,
                            const string& arg, size_t *pid) {
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   wstring wpath;
   Util::UTF8ToWide(path.c_str(), &wpath);
   wpath = L"\"" + wpath + L"\"";
@@ -241,7 +168,7 @@ bool Process::SpawnProcess(const string &path,
       // NOTE: Working directory will be locked by the system.
       // We use system directory to spawn process so that users will not
       // to be aware of any undeletable directory. (http://b/2017482)
-      Util::GetSystemDir(),
+      SystemUtil::GetSystemDir(),
       &si, &pi) != FALSE;
 
   if (create_process_succeeded) {
@@ -322,117 +249,16 @@ bool Process::SpawnProcess(const string &path,
     *pid = tmp_pid;
   }
   return result == 0;
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 }
 
 bool Process::SpawnMozcProcess(
     const string &filename, const string &arg, size_t *pid) {
-  return mozc::Process::SpawnProcess(
-      mozc::Util::JoinPath(mozc::Util::GetServerDirectory(), filename),
+  return Process::SpawnProcess(
+      FileUtil::JoinPath(SystemUtil::GetServerDirectory(), filename),
       arg, pid);
 }
 
-#ifdef OS_WINDOWS
-Process::SecurityInfo::SecurityInfo()
-    : primary_level(sandbox::USER_LOCKDOWN),
-      impersonation_level(sandbox::USER_LOCKDOWN),
-      job_level(sandbox::JOB_LOCKDOWN),
-      integrity_level(sandbox::INTEGRITY_LEVEL_SYSTEM),
-      allow_ui_operation(false),
-      in_system_dir(false),
-      creation_flags(0) {}
-
-bool Process::SpawnProcessAs(const string &path,
-                             const string &arg,
-                             const Process::SecurityInfo &info,
-                             DWORD *pid) {
-  wstring wpath;
-  Util::UTF8ToWide(path.c_str(), &wpath);
-  wpath = L"\"" + wpath + L"\"";
-  if (!arg.empty()) {
-    wstring warg;
-    Util::UTF8ToWide(arg.c_str(), &warg);
-    wpath += L" ";
-    wpath += warg;
-  }
-
-  scoped_array<wchar_t> wpath2(new wchar_t[wpath.size() + 1]);
-  if (0 != wcscpy_s(wpath2.get(), wpath.size() + 1, wpath.c_str())) {
-    return false;
-  }
-
-  HANDLE job_handle = 0;
-  // Create a Job inside restricted sandbox environment.
-  // Child job inherits the environment of the caller.
-  // (http://b/2048754)
-  const DWORD err_code = sandbox::StartRestrictedProcessInJob(
-      wpath2.get(),
-      info.primary_level,
-      info.impersonation_level,
-      info.job_level,
-      // If info.in_system_dir is true, we set the initial directory of the
-      // process to system directory.
-      info.in_system_dir ? Util::GetSystemDir() : NULL,
-      info.integrity_level,
-      info.allow_ui_operation ? JOB_OBJECT_UILIMIT_ALL : 0,
-      info.creation_flags,
-      &job_handle,
-      pid);
-
-  // Remove JOB_OBJECT_LIMIT_KILL_ON_JOB_CLOSE to allow the
-  // child process to continue running even after the parent is terminated
-  // TODO(taku): better to add new paramenter to pass
-  // JOB_OBJECT_LIMIT_KILL_ON_JOB_CLOSE
-  if (NULL != job_handle) {
-    JOBOBJECT_EXTENDED_LIMIT_INFORMATION jeli;
-    if (!::QueryInformationJobObject(job_handle,
-                                     JobObjectExtendedLimitInformation,
-                                     &jeli, sizeof(jeli), NULL)) {
-      LOG(ERROR) << "QueryInformationJobObject failed: "
-                 << ::GetLastError();
-      ::CloseHandle(job_handle);
-      return false;
-    }
-
-    jeli.BasicLimitInformation.LimitFlags
-        &= ~JOB_OBJECT_LIMIT_KILL_ON_JOB_CLOSE;
-    if (!::SetInformationJobObject(job_handle,
-                                   JobObjectExtendedLimitInformation,
-                                   &jeli, sizeof(jeli))) {
-      LOG(ERROR) << "SetQueryInformationJobObject failed: "
-                 << ::GetLastError();
-      ::CloseHandle(job_handle);
-      return false;
-    }
-    ::CloseHandle(job_handle);
-  }
-
-  LOG_IF(ERROR, err_code != ERROR_SUCCESS) <<
-      "sandbox::StartRestrictedProcessInJob failed: " << ::GetLastError();
-
-  return ERROR_SUCCESS == err_code;
-}
-#endif  // OS_WINDOWS
-
-bool Process::BeginBackgroundProcessMode(
-    ProsessPriorityClass *previous_mode) {
-#ifdef OS_WINDOWS
-  return BeginBackgroundProcessModeWin32(previous_mode);
-#else  // OS_WINDOWS
-  // TODO(yukawa): Implement for MacOS and Linux
-  return true;
-#endif
-}
-bool Process::EndBackgroundProcessMode(
-    const ProsessPriorityClass *previous_mode) {
-#ifdef OS_WINDOWS
-  return EndBackgroundProcessModeWin32(previous_mode);
-#else  // OS_WINDOWS
-  // TODO(yukawa): Implement for MacOS and Linux
-  return true;
-#endif
-}
-
 bool Process::WaitProcess(size_t pid, int timeout) {
   if (pid == 0) {
     LOG(WARNING) << "pid is 0. ignored";
@@ -444,7 +270,7 @@ bool Process::WaitProcess(size_t pid, int timeout) {
     return false;
   }
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   DWORD processe_id = static_cast<DWORD>(pid);
   ScopedHandle handle(::OpenProcess(SYNCHRONIZE, FALSE, processe_id));
   if (handle.get() == NULL) {
@@ -490,7 +316,7 @@ bool Process::IsProcessAlive(size_t pid, bool default_result) {
     return default_result;
   }
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   {
     ScopedHandle handle(::OpenProcess(SYNCHRONIZE, FALSE,
                                       static_cast<DWORD>(pid)));
@@ -510,7 +336,7 @@ bool Process::IsProcessAlive(size_t pid, bool default_result) {
 
   return default_result;  // unknown
 
-#else  // OS_WINDOWS
+#else  // OS_WIN
   const int kSig = 0;
   if (::kill(static_cast<pid_t>(pid), kSig) == -1) {
     if (errno == EPERM || errno == EINVAL) {
@@ -521,11 +347,11 @@ bool Process::IsProcessAlive(size_t pid, bool default_result) {
   }
 
   return true;
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 }
 
 bool Process::IsThreadAlive(size_t thread_id, bool default_result) {
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   if (thread_id == 0) {
     return default_result;
   }
@@ -549,7 +375,7 @@ bool Process::IsThreadAlive(size_t thread_id, bool default_result) {
 
   return default_result;  // unknown
 
-#else   // OS_WINDOWS
+#else   // OS_WIN
   // On Linux/Mac, there is no way to check the status of thread id from
   // other process.
   return default_result;
@@ -564,22 +390,20 @@ bool Process::LaunchErrorMessageDialog(const string &error_type) {
   }
 #endif  // OS_MACOSX
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   const string arg = "--mode=error_message_dialog --error_type=" + error_type;
   size_t pid = 0;
-  if (!Process::SpawnProcess(
-          Util::JoinPath(Util::GetServerDirectory(), kMozcTool), arg, &pid)) {
+  if (!Process::SpawnProcess(SystemUtil::GetToolPath(), arg, &pid)) {
     LOG(ERROR) << "cannot launch " << kMozcTool;
     return false;
   }
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 
 #ifdef OS_LINUX
   const char kMozcTool[] = "mozc_tool";
   const string arg = "--mode=error_message_dialog --error_type=" + error_type;
   size_t pid = 0;
-  if (!Process::SpawnProcess(
-          Util::JoinPath(Util::GetServerDirectory(), kMozcTool), arg, &pid)) {
+  if (!Process::SpawnProcess(SystemUtil::GetToolPath(), arg, &pid)) {
     LOG(ERROR) << "cannot launch " << kMozcTool;
     return false;
   }
