@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -32,22 +32,28 @@
 
 #include "converter/candidate_filter.h"
 
-#include <limits.h>
 #include <algorithm>
+#include <climits>
 #include <map>
 #include <set>
-#include <vector>
+#include <string>
 #include <utility>
+#include <vector>
 
-#include "base/util.h"
+#include "base/base.h"
+#include "base/logging.h"
 #include "base/singleton.h"
+#include "base/util.h"
 #include "converter/node.h"
-#include "converter/pos_matcher.h"
 #include "converter/segments.h"
+#include "dictionary/pos_matcher.h"
+#include "dictionary/suppression_dictionary.h"
+#include "prediction/suggestion_filter.h"
 
 namespace mozc {
-
+namespace converter {
 namespace {
+
 const size_t kMaxCandidatesSize = 200;   // how many candidates we expand
 
 // Currently, the cost (logprob) is calcurated as cost = -500 * log(prob).
@@ -73,13 +79,22 @@ const int   kMinCost                 = 100;
 const int   kCostOffset              = 6907;
 const int   kStructureCostOffset     = 3453;
 const int   kMinStructureCostOffset  = 1151;
-const int32 kNoFilterRank            = 3;
-const int32 kNoFilterIfSameIdRank    = 10;
 const int32 kStopEnmerationCacheSize = 15;
-}
 
-CandidateFilter::CandidateFilter()
-    : top_candidate_(NULL) {}
+}  // anonymous namespace
+
+CandidateFilter::CandidateFilter(
+    const SuppressionDictionary *suppression_dictionary,
+    const POSMatcher *pos_matcher,
+    const SuggestionFilter *suggestion_filter)
+    : suppression_dictionary_(suppression_dictionary),
+      pos_matcher_(pos_matcher),
+      suggestion_filter_(suggestion_filter),
+      top_candidate_(NULL) {
+  CHECK(suppression_dictionary_);
+  CHECK(pos_matcher_);
+  CHECK(suggestion_filter_);
+}
 
 CandidateFilter::~CandidateFilter() {}
 
@@ -89,15 +104,55 @@ void CandidateFilter::Reset() {
 }
 
 CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
+    const string &original_key,
     const Segment::Candidate *candidate,
-    const vector<const Node *> nodes) {
+    const vector<const Node *> &nodes,
+    Segments::RequestType request_type) {
   DCHECK(candidate);
 
+  // Filtering by the suggestion filter, which is applied only for the
+  // PREDICTION and SUGGESTION modes.
+  switch (request_type) {
+    case Segments::PREDICTION:
+      // In the PREDICTION mode, the suggestion filter is not applied and the
+      // same filtering rule as the CONVERSION mode is used because the
+      // PREDICTION is triggered by user action (hitting tab keys), i.e.,
+      // prediction candidates are not automatically shown to users. On the
+      // contrary, since a user hit tab keys to run prediction, even unfavorable
+      // words might be what the user wants to type.  Therefore, filtering rule
+      // is relaxed for the PREDICTION mode: we don't apply the suggestion
+      // filter if the user input key is exactly the same as candidate's.
+      if (original_key == candidate->key) {
+        break;
+      }
+      FALLTHROUGH_INTENDED;
+    case Segments::SUGGESTION:
+      // In contrast to the PREDICTION mode, the SUGGESTION is triggered without
+      // any user actions, i.e., suggestion candidates are automatically
+      // displayed to users.  Therefore, it's better to filter unfavorable words
+      // in this mode.
+      CHECK(suggestion_filter_);
+      if (suggestion_filter_->IsBadSuggestion(candidate->value)) {
+        return BAD_CANDIDATE;
+      }
+      // TODO(noriyukit): In the implementation below, the possibility remains
+      // that multiple nodes constitute bad candidates. For stronger filtering,
+      // we may want to check all the possibilities.
+      for (size_t i = 0; i < nodes.size(); ++i) {
+        if (suggestion_filter_->IsBadSuggestion(nodes[i]->value)) {
+          return BAD_CANDIDATE;
+        }
+      }
+      break;
+    default:
+      break;
+  }
+
   // In general, the cost of constrained node tends to be overestimated.
   // If the top candidate has constrained node, we skip the main body
   // of CandidateFilter, meaning that the node is not treated as the top
   // node for CandidateFilter.
-  if (candidate->learning_type & Segment::Candidate::CONTEXT_SENSITIVE) {
+  if (candidate->attributes & Segment::Candidate::CONTEXT_SENSITIVE) {
     return CandidateFilter::GOOD_CANDIDATE;
   }
 
@@ -108,6 +163,11 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
 
   CHECK(top_candidate_);
 
+  // Don't remove duplications if USER_DICTIONARY.
+  if (candidate->attributes & Segment::Candidate::USER_DICTIONARY) {
+    return CandidateFilter::GOOD_CANDIDATE;
+  }
+
   // too many candidates size
   if (candidate_size + 1 >= kMaxCandidatesSize) {
     return CandidateFilter::STOP_ENUMERATION;
@@ -120,6 +180,28 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
 
   CHECK(!nodes.empty());
 
+  // "短縮よみ" must only have 1 node.
+  if (pos_matcher_->IsIsolatedWord(nodes[0]->lid) &&
+      (nodes.size() > 1 ||
+       nodes[0]->prev == NULL ||
+       nodes[0]->prev->node_type == Node::NOR_NODE ||
+       nodes[0]->prev->node_type == Node::CON_NODE ||
+       nodes[0]->next == NULL ||
+       nodes[0]->next->node_type == Node::NOR_NODE ||
+       nodes[0]->next->node_type == Node::CON_NODE)) {
+    return CandidateFilter::BAD_CANDIDATE;
+  }
+
+  // Remove "抑制単語" just in case.
+  if (suppression_dictionary_->SuppressEntry(candidate->key,
+                                             candidate->value) ||
+      (candidate->key != candidate->content_key &&
+       candidate->value != candidate->content_value &&
+       suppression_dictionary_->SuppressEntry(candidate->content_key,
+                                              candidate->content_value))) {
+    return CandidateFilter::BAD_CANDIDATE;
+  }
+
   // The candidate consists of only one token
   if (nodes.size() == 1) {
     VLOG(1) << "don't filter single segment";
@@ -132,19 +214,46 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
     return CandidateFilter::GOOD_CANDIDATE;
   }
 
+  // don't drop lid/rid are the same as those
+  // of top candidate.
+  // http://b/issue?id=4285213
+  if (top_candidate_->structure_cost == 0 &&
+      candidate->lid == top_candidate_->lid &&
+      candidate->rid == top_candidate_->rid) {
+    VLOG(1) << "don't filter lid/rid are the same";
+    return CandidateFilter::GOOD_CANDIDATE;
+  }
+
+  // "好かっ|たり" vs  "良かっ|たり" have same non_content_value.
+  // "良かっ|たり" is also a good candidate but it is not the top candidate.
+  if (top_candidate_ != candidate &&
+      top_candidate_->content_value != top_candidate_->value &&
+      (top_candidate_->value.compare(
+          top_candidate_->content_value.size(),
+          top_candidate_->value.size() - top_candidate_->content_value.size(),
+          candidate->value,
+          candidate->content_value.size(),
+          candidate->value.size() - candidate->content_value.size()) == 0)) {
+    VLOG(1) << "don't filter if non-content value are the same";
+    return CandidateFilter::GOOD_CANDIDATE;
+  }
+
   // Check Katakana transliterations
-  {
-    const bool is_top_katakana_t13n =
-        Segment::IsKatakanaT13NValue(nodes[0]->value);
+  // Skip this check when the conversion mode is real-time;
+  // otherwise this ruins the whole sentence
+  // that starts with alphabets.
+  if (!(candidate->attributes & Segment::Candidate::REALTIME_CONVERSION)) {
+    const bool is_top_english_t13n =
+        Util::IsEnglishTransliteration(nodes[0]->value);
     for (size_t i = 1; i < nodes.size(); ++i) {
-      // KatakanaT13N must be the prefix of the candidate.
-      if (Segment::IsKatakanaT13NValue(nodes[i]->value)) {
+      // EnglishT13N must be the prefix of the candidate.
+      if (Util::IsEnglishTransliteration(nodes[i]->value)) {
         return CandidateFilter::BAD_CANDIDATE;
       }
       // nodes[1..] are non-functional candidates.
       // In other words, the node just after KatakanaT13n candidate should
       // be a functional word.
-      if (is_top_katakana_t13n && !POSMatcher::IsFunctional(nodes[i]->lid)) {
+      if (is_top_english_t13n && !pos_matcher_->IsFunctional(nodes[i]->lid)) {
         return CandidateFilter::BAD_CANDIDATE;
       }
     }
@@ -165,13 +274,13 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
   }
 
   // if candidate starts with prefix "お", we'd like to demote
-  // the candidate if the rank of the candidate is below 3.
+  // the candidate if the rank of the candidate is below 1 (0-origin).
   // This is a temporal workaround for fixing "おそう" => "御|総"
   // TODO(taku): remove it after intorducing a word clustering for noun.
-  if (candidate_size >= 3 && nodes.size() > 1 &&
+  if (candidate_size >= 1 && nodes.size() > 1 &&
       nodes[0]->lid == nodes[0]->rid &&
-      POSMatcher::IsNounPrefix(nodes[0]->lid)) {
-    VLOG(1) << "removing noisy prefix pattern";
+      pos_matcher_->IsWeakCompoundPrefix(nodes[0]->lid)) {
+    VLOG(2) << "removing noisy prefix pattern";
     return CandidateFilter::BAD_CANDIDATE;
   }
 
@@ -180,8 +289,8 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
   // We basically ignores the cost threadshould. Filter candidate
   // only with StructureCost
   int cost_offset = kCostOffset;
-  if (candidate->lid == POSMatcher::GetLastNameId() ||
-      candidate->lid == POSMatcher::GetFirstNameId()) {
+  if (candidate->lid == pos_matcher_->GetLastNameId() ||
+      candidate->lid == pos_matcher_->GetFirstNameId()) {
     cost_offset = INT_MAX;
   }
 
@@ -213,11 +322,17 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
   }
 
   // Filters out candidates with higher cost structure.
-  if (top_structure_cost + kStructureCostOffset < candidate->structure_cost) {
+  if (max(top_structure_cost, kMinStructureCostOffset) + kStructureCostOffset <
+      candidate->structure_cost) {
     // We don't stop enumeration here. Just drops high cost structure
     // looks enough.
+    // |top_structure_cost| can be so small especially for compound or
+    // web dictionary entries.
+    // For avoiding over filtering, we use kMinStructureCostOffset if
+    // |top_structure_cost| is small.
     VLOG(2) << "structure cost is invalid:  "
-            << candidate->value << " " << candidate->structure_cost
+              << candidate->value << " " << candidate->content_value << " "
+              << candidate->structure_cost
             << " " << candidate->cost;
     return CandidateFilter::BAD_CANDIDATE;
   }
@@ -226,14 +341,26 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
 }
 
 CandidateFilter::ResultType CandidateFilter::FilterCandidate(
+    const string &original_key,
     const Segment::Candidate *candidate,
-    const vector<const Node *> nodes) {
-  const CandidateFilter::ResultType result =
-      FilterCandidateInternal(candidate, nodes);
-  if (result != CandidateFilter::GOOD_CANDIDATE) {
+    const vector<const Node *> &nodes,
+    Segments::RequestType request_type) {
+  if (request_type == Segments::REVERSE_CONVERSION) {
+    // In reverse conversion, only remove duplicates because the filtering
+    // criteria of FilterCandidateInternal() are completely designed for
+    // (forward) conversion.
+    const bool inserted = seen_.insert(candidate->value).second;
+    return inserted ? GOOD_CANDIDATE : BAD_CANDIDATE;
+  } else {
+    const ResultType result = FilterCandidateInternal(original_key, candidate,
+                                                      nodes, request_type);
+    if (result != GOOD_CANDIDATE) {
+      return result;
+    }
+    seen_.insert(candidate->value);
     return result;
   }
-  seen_.insert(candidate->value);
-  return result;
 }
+
+}  // namespace converter
 }  // namespace mozc
