@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,11 +29,9 @@
 
 #include <string>
 
-#include "composer/composer.h"
-#include "composer/table.h"
+#include "converter/conversion_request.h"
 #include "converter/converter_mock.h"
 #include "converter/segments.h"
-#include "dictionary/dictionary_interface.h"
 #include "testing/base/public/googletest.h"
 #include "testing/base/public/gunit.h"
 
@@ -51,17 +49,12 @@ void SetSegments(Segments *segments, const string &cand_value) {
   meta_cand->Init();
   meta_cand->value = "TestT13N";
 }
-}
+}  // namespace
 
 class ConverterMockTest : public testing::Test {
- public:
-  void SetUp() {
+ protected:
+  virtual void SetUp() {
     mock_.reset(new ConverterMock);
-    ConverterFactory::SetConverter(mock_.get());
-  }
-
-  void TearDown() {
-    ConverterFactory::SetConverter(NULL);
   }
 
   ConverterMock *GetMock() {
@@ -72,20 +65,8 @@ class ConverterMockTest : public testing::Test {
   scoped_ptr<ConverterMock> mock_;
 };
 
-// This is not a test of ConverterMock but a test of ConverterFactory.
-// TODO(toshiyuki): move this test case to converter_test.cc
-TEST_F(ConverterMockTest, SetConverter) {
-  ConverterMock mock;
-  ConverterInterface *converter = ConverterFactory::GetConverter();
-  EXPECT_NE(&mock, converter);
-
-  ConverterFactory::SetConverter(&mock);
-  ConverterInterface *converter_mock = ConverterFactory::GetConverter();
-  EXPECT_EQ(&mock, converter_mock);
-}
-
 TEST_F(ConverterMockTest, CopySegment) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments output, expect;
   SetSegments(&expect, "StartConvert");
@@ -102,28 +83,18 @@ TEST_F(ConverterMockTest, CopySegment) {
   EXPECT_EQ("TestT13N", seg.meta_candidate(0).value);
 }
 
-TEST_F(ConverterMockTest, SetStartConvert) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+TEST_F(ConverterMockTest, SetStartConversion) {
+  ConverterInterface *converter = GetMock();
 
   Segments output, expect;
-  SetSegments(&expect, "StartConvert");
+  SetSegments(&expect, "StartConversion");
   GetMock()->SetStartConversion(&expect, true);
   EXPECT_TRUE(converter->StartConversion(&output, "dummy"));
   EXPECT_EQ(expect.DebugString(), output.DebugString());
 }
 
-TEST_F(ConverterMockTest, SetStartConvertWithComposer) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
-
-  Segments output, expect;
-  SetSegments(&expect, "StartConvertWithComposer");
-  GetMock()->SetStartConversionWithComposer(&expect, true);
-  EXPECT_TRUE(converter->StartConversionWithComposer(&output, NULL));
-  EXPECT_EQ(expect.DebugString(), output.DebugString());
-}
-
 TEST_F(ConverterMockTest, SetStartReverseConvert) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments output, expect;
   SetSegments(&expect, "StartReverseConvert");
@@ -133,7 +104,7 @@ TEST_F(ConverterMockTest, SetStartReverseConvert) {
 }
 
 TEST_F(ConverterMockTest, SetStartPrediction) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments output, expect;
   SetSegments(&expect, "StartPrediction");
@@ -143,7 +114,7 @@ TEST_F(ConverterMockTest, SetStartPrediction) {
 }
 
 TEST_F(ConverterMockTest, SetStartSuggestion) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments output, expect;
   SetSegments(&expect, "StartSuggestion");
@@ -152,18 +123,39 @@ TEST_F(ConverterMockTest, SetStartSuggestion) {
   EXPECT_EQ(expect.DebugString(), output.DebugString());
 }
 
+TEST_F(ConverterMockTest, SetStartPartialPrediction) {
+  ConverterInterface *converter = GetMock();
+
+  Segments output, expect;
+  SetSegments(&expect, "StartPartialPrediction");
+  GetMock()->SetStartPartialPrediction(&expect, true);
+  EXPECT_TRUE(converter->StartPartialPrediction(&output, "dummy"));
+  EXPECT_EQ(expect.DebugString(), output.DebugString());
+}
+
+TEST_F(ConverterMockTest, SetStartPartialSuggestion) {
+  ConverterInterface *converter = GetMock();
+
+  Segments output, expect;
+  SetSegments(&expect, "StartPartialSuggestion");
+  GetMock()->SetStartPartialSuggestion(&expect, true);
+  EXPECT_TRUE(converter->StartPartialSuggestion(&output, "dummy"));
+  EXPECT_EQ(expect.DebugString(), output.DebugString());
+}
+
 TEST_F(ConverterMockTest, SetFinishConversion) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments output, expect;
   SetSegments(&expect, "FinishConversion");
   GetMock()->SetFinishConversion(&expect, true);
-  EXPECT_TRUE(converter->FinishConversion(&output));
+  const ConversionRequest default_request;
+  EXPECT_TRUE(converter->FinishConversion(default_request, &output));
   EXPECT_EQ(expect.DebugString(), output.DebugString());
 }
 
 TEST_F(ConverterMockTest, SetCancelConversion) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments output, expect;
   SetSegments(&expect, "CancelConversion");
@@ -173,7 +165,7 @@ TEST_F(ConverterMockTest, SetCancelConversion) {
 }
 
 TEST_F(ConverterMockTest, SetResetConversion) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments output, expect;
   SetSegments(&expect, "ResetConversion");
@@ -183,7 +175,7 @@ TEST_F(ConverterMockTest, SetResetConversion) {
 }
 
 TEST_F(ConverterMockTest, SetCommitSegmentValue) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments output, expect;
   SetSegments(&expect, "CommitSegmentValue");
@@ -193,7 +185,7 @@ TEST_F(ConverterMockTest, SetCommitSegmentValue) {
 }
 
 TEST_F(ConverterMockTest, SetFocusSegmentValue) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments output, expect;
   SetSegments(&expect, "FocusSegmentValue");
@@ -203,7 +195,7 @@ TEST_F(ConverterMockTest, SetFocusSegmentValue) {
 }
 
 TEST_F(ConverterMockTest, SetFreeSegmentValue) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments output, expect;
   SetSegments(&expect, "FreeSegmentValue");
@@ -212,70 +204,41 @@ TEST_F(ConverterMockTest, SetFreeSegmentValue) {
   EXPECT_EQ(expect.DebugString(), output.DebugString());
 }
 
-TEST_F(ConverterMockTest, SetSubmitFirstSegment) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+TEST_F(ConverterMockTest, SetCommitFirstSegment) {
+  ConverterInterface *converter = GetMock();
 
   Segments output, expect;
-  SetSegments(&expect, "SubmitFirstSegment");
-  GetMock()->SetSubmitFirstSegment(&expect, true);
-  EXPECT_TRUE(converter->SubmitFirstSegment(&output, 1));
+  SetSegments(&expect, "CommitFirstSegment");
+  GetMock()->SetCommitFirstSegment(&expect, true);
+  EXPECT_TRUE(converter->CommitFirstSegment(&output, 1));
   EXPECT_EQ(expect.DebugString(), output.DebugString());
 }
 
 TEST_F(ConverterMockTest, SetResizeSegment1) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments output, expect;
   SetSegments(&expect, "ResizeSegment1");
   GetMock()->SetResizeSegment1(&expect, true);
-  EXPECT_TRUE(converter->ResizeSegment(&output, 1, 5));
+  const ConversionRequest default_request;
+  EXPECT_TRUE(converter->ResizeSegment(&output, default_request, 1, 5));
   EXPECT_EQ(expect.DebugString(), output.DebugString());
 }
 
 TEST_F(ConverterMockTest, SetResizeSegment2) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments output, expect;
   SetSegments(&expect, "ResizeSegment2");
   GetMock()->SetResizeSegment2(&expect, true);
   uint8 size_array[] = {1, 2, 3};
-  EXPECT_TRUE(converter->ResizeSegment(&output, 1, 5,
+  const ConversionRequest default_request;
+  EXPECT_TRUE(converter->ResizeSegment(&output, default_request, 1, 5,
                                        size_array, arraysize(size_array)));
   EXPECT_EQ(expect.DebugString(), output.DebugString());
 }
-
-TEST_F(ConverterMockTest, SetSync) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
-
-  GetMock()->SetSync(true);
-  EXPECT_TRUE(converter->Sync());
-
-  GetMock()->SetSync(false);
-  EXPECT_FALSE(converter->Sync());
-}
-
-TEST_F(ConverterMockTest, SetClearUserHistory) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
-
-  GetMock()->SetClearUserHistory(true);
-  EXPECT_TRUE(converter->ClearUserHistory());
-
-  GetMock()->SetClearUserHistory(false);
-  EXPECT_FALSE(converter->ClearUserHistory());
-}
-
-TEST_F(ConverterMockTest, SetClearUserPrediction) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
-
-  GetMock()->SetClearUserPrediction(true);
-  EXPECT_TRUE(converter->ClearUserPrediction());
-
-  GetMock()->SetClearUserPrediction(false);
-  EXPECT_FALSE(converter->ClearUserPrediction());
-}
-
 TEST_F(ConverterMockTest, GetStartConversion) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments input;
   const string input_key = "Key";
@@ -292,26 +255,8 @@ TEST_F(ConverterMockTest, GetStartConversion) {
   EXPECT_EQ(input_key, last_key);
 }
 
-TEST_F(ConverterMockTest, GetStartConversionWithComposer) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
-
-  Segments input;
-  composer::Composer input_composer;
-  SetSegments(&input, "StartConversionWithComposer");
-  const string input_str = input.DebugString();
-  converter->StartConversionWithComposer(&input, &input_composer);
-
-  Segments last_segment;
-  const composer::Composer *last_composer;
-  GetMock()->GetStartConversionWithComposer(&last_segment, &last_composer);
-  const string last_segment_str = last_segment.DebugString();
-
-  EXPECT_EQ(input_str, last_segment_str);
-  EXPECT_EQ(&input_composer, last_composer);
-}
-
 TEST_F(ConverterMockTest, GetStartReverseConversion) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments input;
   const string input_key = "Key";
@@ -329,7 +274,7 @@ TEST_F(ConverterMockTest, GetStartReverseConversion) {
 }
 
 TEST_F(ConverterMockTest, GetStartPrediction) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments input;
   const string input_key = "Key";
@@ -347,7 +292,7 @@ TEST_F(ConverterMockTest, GetStartPrediction) {
 }
 
 TEST_F(ConverterMockTest, GetStartSuggestion) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments input;
   const string input_key = "Key";
@@ -364,13 +309,50 @@ TEST_F(ConverterMockTest, GetStartSuggestion) {
   EXPECT_EQ(input_key, last_key);
 }
 
+TEST_F(ConverterMockTest, GetStartPartialPrediction) {
+  ConverterInterface *converter = GetMock();
+
+  Segments input;
+  const string input_key = "Key";
+  SetSegments(&input, "StartPartialPrediction");
+  const string input_str = input.DebugString();
+  converter->StartPartialPrediction(&input, input_key);
+
+  Segments last_segment;
+  string last_key;
+  GetMock()->GetStartPartialPrediction(&last_segment, &last_key);
+  const string last_segment_str = last_segment.DebugString();
+
+  EXPECT_EQ(input_str, last_segment_str);
+  EXPECT_EQ(input_key, last_key);
+}
+
+TEST_F(ConverterMockTest, GetStartPartialSuggestion) {
+  ConverterInterface *converter = GetMock();
+
+  Segments input;
+  const string input_key = "Key";
+  SetSegments(&input, "StartPartialSuggestion");
+  const string input_str = input.DebugString();
+  converter->StartPartialSuggestion(&input, input_key);
+
+  Segments last_segment;
+  string last_key;
+  GetMock()->GetStartPartialSuggestion(&last_segment, &last_key);
+  const string last_segment_str = last_segment.DebugString();
+
+  EXPECT_EQ(input_str, last_segment_str);
+  EXPECT_EQ(input_key, last_key);
+}
+
 TEST_F(ConverterMockTest, GetFinishConversion) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments input;
   SetSegments(&input, "FinishConversion");
   const string input_str = input.DebugString();
-  converter->FinishConversion(&input);
+  ConversionRequest default_request;
+  converter->FinishConversion(default_request, &input);
 
   Segments last_segment;
   GetMock()->GetFinishConversion(&last_segment);
@@ -380,7 +362,7 @@ TEST_F(ConverterMockTest, GetFinishConversion) {
 }
 
 TEST_F(ConverterMockTest, GetCancelConversion) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments input;
   SetSegments(&input, "CancelConversion");
@@ -395,7 +377,7 @@ TEST_F(ConverterMockTest, GetCancelConversion) {
 }
 
 TEST_F(ConverterMockTest, GetResetConversion) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments input;
   SetSegments(&input, "ResetConversion");
@@ -410,7 +392,7 @@ TEST_F(ConverterMockTest, GetResetConversion) {
 }
 
 TEST_F(ConverterMockTest, GetCommitSegmentValue) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments input;
   size_t input_idx = 1;
@@ -431,7 +413,7 @@ TEST_F(ConverterMockTest, GetCommitSegmentValue) {
 }
 
 TEST_F(ConverterMockTest, GetFocusSegmentValue) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments input;
   size_t input_idx = 1;
@@ -452,7 +434,7 @@ TEST_F(ConverterMockTest, GetFocusSegmentValue) {
 }
 
 TEST_F(ConverterMockTest, GetFreeSegmentValue) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments input;
   size_t input_idx = 1;
@@ -469,18 +451,18 @@ TEST_F(ConverterMockTest, GetFreeSegmentValue) {
   EXPECT_EQ(input_idx, last_idx);
 }
 
-TEST_F(ConverterMockTest, GetSubmitFirstSegment) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+TEST_F(ConverterMockTest, GetCommitFirstSegment) {
+  ConverterInterface *converter = GetMock();
 
   Segments input;
   size_t input_idx = 1;
-  SetSegments(&input, "SubmitFirstSegment");
+  SetSegments(&input, "CommitFirstSegment");
   const string input_str = input.DebugString();
-  converter->SubmitFirstSegment(&input, input_idx);
+  converter->CommitFirstSegment(&input, input_idx);
 
   Segments last_segment;
   size_t last_idx;
-  GetMock()->GetSubmitFirstSegment(&last_segment, &last_idx);
+  GetMock()->GetCommitFirstSegment(&last_segment, &last_idx);
   const string last_segment_str = last_segment.DebugString();
 
   EXPECT_EQ(input_str, last_segment_str);
@@ -488,14 +470,16 @@ TEST_F(ConverterMockTest, GetSubmitFirstSegment) {
 }
 
 TEST_F(ConverterMockTest, GetResizeSegment1) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments input;
   size_t input_idx = 1;
   int input_offset = 3;
   SetSegments(&input, "ResizeSegment1");
   const string input_str = input.DebugString();
-  converter->ResizeSegment(&input, input_idx, input_offset);
+  const ConversionRequest default_request;
+  converter->ResizeSegment(
+      &input, default_request, input_idx, input_offset);
 
   Segments last_segment;
   size_t last_idx;
@@ -509,14 +493,15 @@ TEST_F(ConverterMockTest, GetResizeSegment1) {
 }
 
 TEST_F(ConverterMockTest, GetResizeSegment2) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments input;
   size_t input_idx = 1, input_size = 3;
   uint8 input_array[] = {1, 2, 3};
   SetSegments(&input, "ResizeSegment2");
   const string input_str = input.DebugString();
-  converter->ResizeSegment(&input, input_idx, input_size,
+  const ConversionRequest default_request;
+  converter->ResizeSegment(&input, default_request, input_idx, input_size,
                            input_array, arraysize(input_array));
 
   Segments last_segment;
@@ -525,7 +510,7 @@ TEST_F(ConverterMockTest, GetResizeSegment2) {
   size_t last_array_size;
 
   GetMock()->GetResizeSegment2(&last_segment, &last_idx, &last_size,
-                         &last_array, &last_array_size);
+                               &last_array, &last_array_size);
   const string last_segment_str = last_segment.DebugString();
 
   EXPECT_EQ(input_str, last_segment_str);
@@ -538,7 +523,7 @@ TEST_F(ConverterMockTest, GetResizeSegment2) {
 }
 
 TEST_F(ConverterMockTest, DefaultBehavior) {
-  ConverterInterface *converter = ConverterFactory::GetConverter();
+  ConverterInterface *converter = GetMock();
 
   Segments input;
   const string input_key = "Key";
