@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -33,8 +33,12 @@
 
 #include <string>
 
+#include "base/const.h"
+#include "base/file_util.h"
 #include "base/process.h"
 #include "base/run_level.h"
+#include "base/system_util.h"
+#include "base/update_checker.h"
 #include "base/util.h"
 #include "base/version.h"
 
@@ -42,6 +46,10 @@ namespace mozc {
 namespace gui {
 
 namespace {
+#ifdef USE_UPDATE_CHECKER
+const UINT kUpdateCheckMessage = WM_USER;
+#endif  // USE_UPDATE_CHECKER
+
 void defaultLinkActivated(const QString &str) {
   QByteArray utf8 = str.toUtf8();
   Process::OpenBrowser(string(utf8.data(), utf8.length()));
@@ -53,9 +61,8 @@ bool AddLocalPath(string *str) {
   for (size_t i = 0; i < arraysize(filenames); ++i) {
     if (str->find(filenames[i]) != string::npos) {
       string tmp;
-      const string documents_dir = Util::JoinPath(Util::GetServerDirectory(),
-                                                  "documents");
-      const string file_path = Util::JoinPath(documents_dir, filenames[i]);
+      const string file_path =
+          FileUtil::JoinPath(SystemUtil::GetDocumentDirectory(), filenames[i]);
       Util::StringReplace(*str, filenames[i], file_path, false, &tmp);
       *str = tmp;
       return true;
@@ -86,6 +93,14 @@ AboutDialog::AboutDialog(QWidget *parent)
   version_info += Version::GetMozcVersion().c_str();
   version_info += ")";
   version_label->setText(version_info);
+  updateButton->hide();
+#ifdef USE_UPDATE_CHECKER
+  UpdateChecker::CallbackInfo info;
+  info.mesage_receiver_window = winId();
+  info.mesage_id = kUpdateCheckMessage;
+  UpdateChecker::BeginCheck(info);
+#endif  // USE_UPDATE_CHECKER
+
   QPalette palette;
   palette.setColor(QPalette::Window, QColor(236, 233, 216));
   color_frame->setPalette(palette);
@@ -93,9 +108,9 @@ AboutDialog::AboutDialog(QWidget *parent)
 
   // change font size for product name
   QFont font = label->font();
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   font.setPointSize(22);
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 
 #ifdef OS_MACOSX
   font.setPointSize(26);
@@ -135,5 +150,48 @@ void AboutDialog::linkActivated(const QString &link) {
     defaultLinkActivated(link);
   }
 }
-}  // namespace mozc::gui
+
+#ifdef USE_UPDATE_CHECKER
+bool AboutDialog::winEvent(MSG *message, long *result) {
+  if (message != NULL && message->message == kUpdateCheckMessage) {
+    QString version_info("(");
+    version_info += Version::GetMozcVersion().c_str();
+    version_info += ") - ";
+    // TODO(yukawa, team): Make better UI.
+    switch (message->wParam) {
+      case UpdateChecker::UPGRADE_IS_AVAILABLE:
+        version_info += tr("New version is available");
+        if (SystemUtil::IsVistaOrLater()) {
+          if (!RunLevel::IsElevatedByUAC()) {
+            QWindowsStyle style;
+            QIcon vista_icon(style.standardIcon(QStyle::SP_VistaShield));
+            updateButton->setIcon(vista_icon);
+          }
+        }
+        updateButton->show();
+        break;
+      case UpdateChecker::UPGRADE_ALREADY_UP_TO_DATE:
+        version_info += tr("You are using the latest version");
+        break;
+      case UpdateChecker::UPGRADE_ERROR:
+      default:
+        break;
+    }
+    version_label->setText(version_info);
+    *result = 0;
+    return true;
+  }
+
+  return QWidget::winEvent(message, result);
+}
+#endif  // USE_UPDATE_CHECKER
+
+void AboutDialog::updateButtonPushed() {
+  updateButton->setEnabled(false);
+  // Currently, update_dialog is available only on Windows.
+#if defined(OS_WIN)
+  Process::SpawnMozcProcess(mozc::kMozcTool, "--mode=update_dialog");
+#endif  // OS_WIN
+}
+}  // namespace gui
 }  // namespace mozc
