@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -35,18 +35,19 @@
 #include <algorithm>  // for unique
 #include <cctype>
 #include <set>
-#include <string>
 #include <sstream>
+#include <string>
 #include <vector>
-#include "base/base.h"
-#include "base/singleton.h"
 #include "base/config_file_stream.h"
+#include "base/logging.h"
+#include "base/singleton.h"
 #include "base/util.h"
+#include "gui/base/table_util.h"
 #include "gui/config_dialog/combobox_delegate.h"
 #include "gui/config_dialog/keybinding_editor_delegate.h"
 #include "session/commands.pb.h"
-#include "session/key_parser.h"
 #include "session/internal/keymap.h"
+#include "session/key_parser.h"
 // TODO(komatsu): internal files should not be used from external modules.
 
 namespace mozc {
@@ -72,6 +73,7 @@ const char kAbortCommand[] = "Abort";
 const char kInsertCharacterCommand[] = "InsertCharacter";
 const char kIMEOnCommand[] = "IMEOn";
 const char kIMEOffCommand[] = "IMEOff";
+const char kReconvertCommand[] = "Reconvert";
 const char kReportBugCommand[] = "ReportBug";
 // Old command name
 const char kEditInsertCommand[] = "EditInsert";
@@ -93,13 +95,13 @@ class KeyMapValidator {
     invisible_commands_.insert(kReportBugCommand);
     // Old command name.
     invisible_commands_.insert(kEditInsertCommand);
-#if defined(OS_MACOSX) || defined(OS_LINUX)
-    // On Mac/Linux, we cannot customize keybindings for IME ON/OFF
+#if defined(OS_MACOSX)
+    // On Mac, we cannot customize keybindings for IME ON/OFF
     // So we do not show them.
     // TODO(toshiyuki): remove them after implimenting IME ON/OFF for Mac
     invisible_commands_.insert(kIMEOnCommand);
     invisible_commands_.insert(kIMEOffCommand);
-#endif  // OS_MACOSX || OS_LINUX
+#endif  // OS_MACOSX
 
     invisible_modifiers_.insert(mozc::commands::KeyEvent::KEY_DOWN);
     invisible_modifiers_.insert(mozc::commands::KeyEvent::KEY_UP);
@@ -107,7 +109,7 @@ class KeyMapValidator {
     invisible_key_events_.insert(mozc::commands::KeyEvent::KANJI);
     invisible_key_events_.insert(mozc::commands::KeyEvent::ON);
     invisible_key_events_.insert(mozc::commands::KeyEvent::OFF);
-    invisible_key_events_.insert(mozc::commands::KeyEvent::ASCII);
+    invisible_key_events_.insert(mozc::commands::KeyEvent::TEXT_INPUT);
   }
 
   bool IsVisibleKey(const string &key) {
@@ -184,6 +186,7 @@ class KeyMapValidator {
 
     return true;
   }
+
  private:
   set<uint32> invisible_modifiers_;
   set<uint32> invisible_key_events_;
@@ -206,6 +209,7 @@ class KeyMapTableLoader {
     manager.GetAvailableCommandNamePrecomposition(&command_names);
     manager.GetAvailableCommandNameComposition(&command_names);
     manager.GetAvailableCommandNameConversion(&command_names);
+    manager.GetAvailableCommandNameZeroQuerySuggestion(&command_names);
     manager.GetAvailableCommandNameSuggestion(&command_names);
     manager.GetAvailableCommandNamePrediction(&command_names);
     for (set<string>::const_iterator itr = command_names.begin();
@@ -300,7 +304,6 @@ KeyMapEditorDialog::KeyMapEditorDialog(QWidget *parent)
                                                    keybinding_delegate_.get());
   mutable_table_widget()->setItemDelegateForColumn(2,
                                                    commands_delegate_.get());
-  mutable_table_widget()->setEditTriggers(QAbstractItemView::AllEditTriggers);
 
   setWindowTitle(tr("Mozc keymap editor"));
   CHECK(mutable_table_widget());
@@ -364,7 +367,7 @@ bool KeyMapEditorDialog::LoadFromStream(istream *is) {
       continue;
     }
 
-    if (fields[2] == kIMEOnCommand || fields[2] == kIMEOffCommand) {
+    if (fields[2] == kIMEOnCommand || fields[2] == kReconvertCommand) {
       ime_switch_keymap_.insert(line);
     }
 
@@ -390,10 +393,10 @@ bool KeyMapEditorDialog::LoadFromStream(istream *is) {
 bool KeyMapEditorDialog::Update() {
   if (mutable_table_widget()->rowCount() == 0) {
     QMessageBox::warning(this,
-                         tr("Mozc settings"),
+                         windowTitle(),
                          tr("Current keymap table is empty. "
                             "You might want to import a pre-defined "
-                            "keymap table first"));
+                            "keymap table first."));
     return false;
   }
 
@@ -405,12 +408,12 @@ bool KeyMapEditorDialog::Update() {
   *keymap_table = "status\tkey\tcommand\n";
 
   for (int i = 0; i < mutable_table_widget()->rowCount(); ++i) {
-    const string i18n_status =
-        mutable_table_widget()->item(i, 0)->text().toStdString();
-    const string key =
-        mutable_table_widget()->item(i, 1)->text().toStdString();
-    const string i18n_command =
-        mutable_table_widget()->item(i, 2)->text().toStdString();
+    const string &i18n_status =
+        TableUtil::SafeGetItemText(mutable_table_widget(), i, 0).toStdString();
+    const string &key =
+        TableUtil::SafeGetItemText(mutable_table_widget(), i, 1).toStdString();
+    const string &i18n_command =
+        TableUtil::SafeGetItemText(mutable_table_widget(), i, 2).toStdString();
 
     const map<string, string>::const_iterator status_it =
         normalized_status_map_.find(i18n_status);
@@ -430,7 +433,7 @@ bool KeyMapEditorDialog::Update() {
 
     if (!validator->IsVisibleKey(key)) {
       QMessageBox::warning(this,
-                           tr("Mozc settings"),
+                           windowTitle(),
                            (tr("Invalid key:\n%1")
                             .arg(QString::fromUtf8(key.c_str()))));
       return false;
@@ -439,20 +442,20 @@ bool KeyMapEditorDialog::Update() {
     *keymap_table += keymap_line;
     *keymap_table += '\n';
 
-    if (command == kIMEOnCommand || command == kIMEOffCommand) {
+    if (command == kIMEOnCommand || command == kReconvertCommand) {
       new_ime_switch_keymap.insert(keymap_line);
     }
   }
   *keymap_table += invisible_keymap_table_;
 
   if (new_ime_switch_keymap != ime_switch_keymap_) {
-#if defined(OS_WINDOWS) || defined(OS_LINUX)
+#if defined(OS_WIN) || defined(OS_LINUX)
     QMessageBox::information(
         this,
-        tr("Mozc settings"),
-        tr("The keymaps for IME ON/OFF will be "
+        windowTitle(),
+        tr("The keymaps for IME ON and Reconversion will be "
            "applied after new applications."));
-#endif  // OS_WINDOWS || OS_LINUX
+#endif  // OS_WIN || OS_LINUX
     ime_switch_keymap_ = new_ime_switch_keymap;
   }
 
@@ -485,7 +488,7 @@ void KeyMapEditorDialog::OnEditMenuAction(QAction *action) {
         QMessageBox::Ok !=
         QMessageBox::question(
             this,
-            tr("Mozc settings"),
+            windowTitle(),
             tr("Do you want to overwrite the current keymaps?"),
             QMessageBox::Ok | QMessageBox::Cancel,
             QMessageBox::Cancel)) {
@@ -501,7 +504,7 @@ void KeyMapEditorDialog::OnEditMenuAction(QAction *action) {
       const char *keymap_file =
           keymap::KeyMapManager::GetKeyMapFileName(kKeyMaps[import_index]);
       scoped_ptr<istream> ifs(
-          ConfigFileStream::Open(keymap_file));
+          ConfigFileStream::LegacyOpen(keymap_file));
       CHECK(ifs.get() != NULL);  // should never happen
       CHECK(LoadFromStream(ifs.get()));
     }
