@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,13 +29,16 @@
 
 #import "mac/GoogleJapaneseInputController.h"
 
-#import <Cocoa/Cocoa.h>
 #import <Carbon/Carbon.h>
-#import <InputMethodKit/IMKServer.h>
+#import <Cocoa/Cocoa.h>
 #import <InputMethodKit/IMKInputController.h>
+#import <InputMethodKit/IMKServer.h>
 
 #include <unistd.h>
 
+#include <cstdlib>
+#include <set>
+
 #import "mac/GoogleJapaneseInputControllerInterface.h"
 #import "mac/GoogleJapaneseInputServer.h"
 #import "mac/KeyCodeMap.h"
@@ -47,11 +50,11 @@
 #include "base/mutex.h"
 #include "base/process.h"
 #include "base/util.h"
-#include "client/session.h"
+#include "client/client.h"
+#include "config/config.pb.h"
 #include "ipc/ipc.h"
 #include "renderer/renderer_client.h"
 #include "session/commands.pb.h"
-#include "session/config.pb.h"
 #include "session/ime_switch_util.h"
 
 using mozc::commands::Candidates;
@@ -68,13 +71,22 @@ using mozc::config::ImeSwitchUtil;
 using mozc::kProductNameInEnglish;
 using mozc::once_t;
 using mozc::CallOnce;
+using mozc::MacProcess;
 
 namespace {
 // set of bundle IDs of applications on which Mozc should not open urls.
-NSSet *gNoOpenLinkApps = nil;
+const set<string> *gNoOpenLinkApps = NULL;
 // The mapping from the CompositionMode enum to the actual id string
 // of composition modes.
 const map<CompositionMode, NSString *> *gModeIdMap = NULL;
+const set<string> *gNoSelectedRangeApps = NULL;
+const set<string> *gNoDisplayModeSwitchApps = NULL;
+
+// TODO(horo): This value should be get from system configuration.
+//  DoubleClickInterval can be get from NSEvent (MacOSX ver >= 10.6)
+const NSTimeInterval kDoubleTapInterval = 0.5;
+
+const int kMaxSurroundingLength = 20;
 
 NSString *GetLabelForSuffix(const string &suffix) {
   string label = mozc::MacUtil::GetLabelForSuffix(suffix);
@@ -128,6 +140,11 @@ CompositionMode GetCompositionMode(NSString *modeID) {
   return mozc::commands::DIRECT;
 }
 
+bool IsBannedApplication(const set<string>* bundleIdSet,
+                         const string& bundleId) {
+  return bundleIdSet == NULL || bundleId.empty() ||
+      bundleIdSet->find(bundleId) != bundleIdSet->end();
+}
 }  // anonymous namespace
 
 
@@ -137,12 +154,14 @@ CompositionMode GetCompositionMode(NSString *modeID) {
 @synthesize yenSignCharacter = yenSignCharacter_;
 @synthesize mode = mode_;
 @synthesize rendererCommand = rendererCommand_;
-- (mozc::client::SessionInterface *)session {
-  return session_;
+@synthesize replacementRange = replacementRange_;
+@synthesize imkClientForTest = imkClientForTest_;
+- (mozc::client::ClientInterface *)mozcClient {
+  return mozcClient_;
 }
-- (void)setSession:(mozc::client::SessionInterface *)newSession {
-  delete session_;
-  session_ = newSession;
+- (void)setMozcClient:(mozc::client::ClientInterface *)newMozcClient {
+  delete mozcClient_;
+  mozcClient_ = newMozcClient;
 }
 - (mozc::renderer::RendererInterface *)renderer {
   return candidateController_;
@@ -165,22 +184,27 @@ CompositionMode GetCompositionMode(NSString *modeID) {
     return self;
   }
   keyCodeMap_ = [[KeyCodeMap alloc] init];
-  clientBundle_ = [[inputClient bundleIdentifier] copy];
+  clientBundle_ = new(nothrow) string;
+  replacementRange_ = NSMakeRange(NSNotFound, 0);
   originalString_ = [[NSMutableString alloc] init];
   composedString_ = [[NSMutableAttributedString alloc] init];
   cursorPosition_ = NSNotFound;
   mode_ = mozc::commands::DIRECT;
   checkInputMode_ = YES;
+  suppressSuggestion_ = NO;
   yenSignCharacter_ = mozc::config::Config::YEN_SIGN;
   candidateController_ = new(nothrow) mozc::renderer::RendererClient;
   rendererCommand_ = new(nothrow)RendererCommand;
-  session_ = new(nothrow) mozc::client::Session();
-  server_ = reinterpret_cast<id<ServerCallback> >(server);
+  mozcClient_ = mozc::client::ClientFactory::NewClient();
+  imkServer_ = reinterpret_cast<id<ServerCallback> >(server);
+  imkClientForTest_ = nil;
+  lastKeyDownTime_ = 0;
+  lastKeyCode_ = 0;
 
   // We don't check the return value of NSBundle because it fails during tests.
   [NSBundle loadNibNamed:@"Config" owner:self];
   if (!originalString_ || !composedString_ || !candidateController_ ||
-      !rendererCommand_ || !session_) {
+      !rendererCommand_ || !mozcClient_ || !clientBundle_) {
     [self release];
     self = nil;
   } else {
@@ -191,6 +215,8 @@ CompositionMode GetCompositionMode(NSString *modeID) {
       delete candidateController_;
       candidateController_ = NULL;
     }
+    [self setupClientBundle:inputClient];
+    [self setupCapability];
     RendererCommand::ApplicationInfo *applicationInfo =
         rendererCommand_->mutable_application_info();
     applicationInfo->set_process_id(::getpid());
@@ -207,33 +233,69 @@ CompositionMode GetCompositionMode(NSString *modeID) {
   [keyCodeMap_ release];
   [originalString_ release];
   [composedString_ release];
-  [clientBundle_ release];
+  [imkClientForTest_ release];
+  delete clientBundle_;
   delete candidateController_;
-  delete session_;
+  delete mozcClient_;
   delete rendererCommand_;
   DLOG(INFO) << "dealloc server";
   [super dealloc];
 }
 
+- (id)client {
+  if (imkClientForTest_) {
+    return imkClientForTest_;
+  }
+  return [super client];
+}
+
 - (NSMenu*)menu {
   return menu_;
 }
 
 + (void)initializeConstants {
-  // should not open links during screensaver.
-  gNoOpenLinkApps =
-      [NSSet setWithObjects:@"com.apple.securityagent", nil];
+  set<string> *noOpenlinkApps = new(nothrow) set<string>;
+  if (noOpenlinkApps) {
+    // should not open links during screensaver.
+    noOpenlinkApps->insert("com.apple.securityagent");
+    gNoOpenLinkApps = noOpenlinkApps;
+  }
 
   map<CompositionMode, NSString *> *newMap =
-      new map<CompositionMode, NSString *>;
-  (*newMap)[mozc::commands::DIRECT] = GetLabelForSuffix("Roman");
-  (*newMap)[mozc::commands::HIRAGANA] = GetLabelForSuffix("base");
-  (*newMap)[mozc::commands::FULL_KATAKANA] = GetLabelForSuffix("Katakana");
-  (*newMap)[mozc::commands::HALF_ASCII] = GetLabelForSuffix("Roman");
-  (*newMap)[mozc::commands::FULL_ASCII] = GetLabelForSuffix("FullWidthRoman");
-  (*newMap)[mozc::commands::HALF_KATAKANA] =
-      GetLabelForSuffix("FullWidthRoman");
-  gModeIdMap = newMap;
+      new(nothrow) map<CompositionMode, NSString *>;
+  if (newMap) {
+    (*newMap)[mozc::commands::DIRECT] = GetLabelForSuffix("Roman");
+    (*newMap)[mozc::commands::HIRAGANA] = GetLabelForSuffix("base");
+    (*newMap)[mozc::commands::FULL_KATAKANA] = GetLabelForSuffix("Katakana");
+    (*newMap)[mozc::commands::HALF_ASCII] = GetLabelForSuffix("Roman");
+    (*newMap)[mozc::commands::FULL_ASCII] = GetLabelForSuffix("FullWidthRoman");
+    (*newMap)[mozc::commands::HALF_KATAKANA] =
+        GetLabelForSuffix("FullWidthRoman");
+    gModeIdMap = newMap;
+  }
+
+  set<string> *noSelectedRangeApps = new(nothrow) set<string>;
+  if (noSelectedRangeApps) {
+    // Do not call selectedRange: method for the following
+    // applications because it could lead to application crash.
+    noSelectedRangeApps->insert("com.microsoft.Excel");
+    noSelectedRangeApps->insert("com.microsoft.Powerpoint");
+    noSelectedRangeApps->insert("com.microsoft.Word");
+    gNoSelectedRangeApps = noSelectedRangeApps;
+  }
+
+  // Do not call selectInputMode: method for the following
+  // applications because it could cause some unexpected behavior.
+  // MS-Word: When the display mode goes to ASCII but there is no
+  // compositions, it goes to direct input mode instead of Half-ASCII
+  // mode.  When the first composition character is alphanumeric (such
+  // like pressing Shift-A at first), that character is directly
+  // inserted into application instead of composition starting "A".
+  set<string> *noDisplayModeSwitchApps = new(nothrow) set<string>;
+  if (noDisplayModeSwitchApps) {
+    noDisplayModeSwitchApps->insert("com.microsoft.Word");
+    gNoDisplayModeSwitchApps = noDisplayModeSwitchApps;
+  }
 }
 
 #pragma mark IMKStateSetting Protocol
@@ -244,18 +306,28 @@ CompositionMode GetCompositionMode(NSString *modeID) {
 
 - (void)activateServer:(id)sender {
   [super activateServer:sender];
-  [clientBundle_ release];
-  clientBundle_ = [[sender bundleIdentifier] copy];
   checkInputMode_ = YES;
   if (rendererCommand_->visible() && candidateController_) {
     candidateController_->ExecCommand(*rendererCommand_);
   }
-  [server_ setCurrentController:self];
-  DLOG(INFO) << [[NSString stringWithFormat:
-                             @"%s client (%@): activated for %@",
-                           kProductNameInEnglish, self, sender] UTF8String];
-  DLOG(INFO) << [[NSString stringWithFormat:
-                             @"sender bundleID: %@", clientBundle_] UTF8String];
+  [self handleConfig];
+  [imkServer_ setCurrentController:self];
+
+  string window_name, window_owner;
+  if (mozc::MacUtil::GetFrontmostWindowNameAndOwner(&window_name,
+                                                    &window_owner)) {
+    DLOG(INFO) << "frontmost window name: \"" << window_name << "\" "
+               << "owner: \"" << window_owner << "\"";
+    if (mozc::MacUtil::IsSuppressSuggestionWindow(window_name, window_owner)) {
+      suppressSuggestion_ = YES;
+    } else {
+      suppressSuggestion_ = NO;
+    }
+  }
+
+  DLOG(INFO) << kProductNameInEnglish << " client (" << self
+             << "): activated for " << sender;
+  DLOG(INFO) << "sender bundleID: " << *clientBundle_;
 }
 
 - (void)deactivateServer:(id)sender {
@@ -266,11 +338,9 @@ CompositionMode GetCompositionMode(NSString *modeID) {
   if (candidateController_) {
     candidateController_->ExecCommand(clearCommand);
   }
-  DLOG(INFO) << [[NSString stringWithFormat:
-                             @"%s client (%@): activated for %@",
-                           kProductNameInEnglish, self, sender] UTF8String];
-  DLOG(INFO) << [[NSString stringWithFormat:
-                             @"sender bundleID: %@", clientBundle_] UTF8String];
+  DLOG(INFO) << kProductNameInEnglish << " client (" << self
+             << "): deactivated";
+  DLOG(INFO) << "sender bundleID: " << *clientBundle_;
   [super deactivateServer:sender];
 }
 
@@ -289,9 +359,55 @@ CompositionMode GetCompositionMode(NSString *modeID) {
   }
 
   [self switchMode:new_mode client:sender];
+  [self handleConfig];
   [super setValue:value forTag:tag client:sender];
 }
 
+
+#pragma mark internal methods
+
+- (void)handleConfig {
+  // Get the config and set client-side behaviors
+  Config config;
+  if (!mozcClient_->GetConfig(&config)) {
+    LOG(ERROR) << "Cannot obtain the current config";
+    return;
+  }
+
+  InputMode input_mode = ASCII;
+  if (config.preedit_method() == Config::KANA) {
+    input_mode = KANA;
+  }
+  [keyCodeMap_ setInputMode:input_mode];
+  yenSignCharacter_ = config.yen_sign_character();
+
+  if (config.use_japanese_layout()) {
+    // Apple does not have "Japanese" layout actually -- here sets
+    // "US" layout, which means US-ASCII layout or JIS layout
+    // depending on which type of keyboard is actually connected.
+    [[self client] overrideKeyboardWithKeyboardNamed:@"com.apple.keylayout.US"];
+  }
+}
+
+- (void)setupClientBundle:(id)sender {
+  NSString *bundleIdentifier = [sender bundleIdentifier];
+  if (bundleIdentifier != nil && [bundleIdentifier length] > 0) {
+    clientBundle_->assign([bundleIdentifier UTF8String]);
+  }
+}
+
+- (void)setupCapability {
+  Capability capability;
+
+  if (IsBannedApplication(gNoSelectedRangeApps, *clientBundle_)) {
+    capability.set_text_deletion(Capability::NO_TEXT_DELETION_CAPABILITY);
+  } else {
+    capability.set_text_deletion(Capability::DELETE_PRECEDING_TEXT);
+  }
+
+  mozcClient_->set_client_capability(capability);
+}
+
 // Mode changes to direct and clean up the status.
 - (void)switchModeToDirect:(id)sender {
   mode_ = mozc::commands::DIRECT;
@@ -299,13 +415,9 @@ CompositionMode GetCompositionMode(NSString *modeID) {
   KeyEvent keyEvent;
   Output output;
   keyEvent.set_special_key(mozc::commands::KeyEvent::OFF);
-  session_->SendKey(keyEvent, &output);
+  mozcClient_->SendKey(keyEvent, &output);
   if (output.has_result()) {
-    NSString *result_text =
-        [NSString
-            stringWithUTF8String:output.result().value().c_str()];
-    [sender insertText:result_text
-      replacementRange:NSMakeRange(NSNotFound, 0)];
+    [self commitText:output.result().value().c_str() client:sender];
   }
   if ([composedString_ length] > 0) {
     [self updateComposedString:NULL];
@@ -321,7 +433,7 @@ CompositionMode GetCompositionMode(NSString *modeID) {
     KeyEvent keyEvent;
     Output output;
     keyEvent.set_special_key(mozc::commands::KeyEvent::ON);
-    session_->SendKey(keyEvent, &output);
+    mozcClient_->SendKey(keyEvent, &output);
   }
 
   if (mode_ != new_mode) {
@@ -331,12 +443,15 @@ CompositionMode GetCompositionMode(NSString *modeID) {
     command.set_type(mozc::commands::SessionCommand::SWITCH_INPUT_MODE);
     command.set_composition_mode(new_mode);
     Output output;
-    session_->SendCommand(command, &output);
+    mozcClient_->SendCommand(command, &output);
     mode_ = new_mode;
   }
 }
 
 - (void)switchMode:(CompositionMode)new_mode client:(id)sender {
+  if (mode_ == new_mode) {
+    return;
+  }
   if (mode_ != mozc::commands::DIRECT && new_mode == mozc::commands::DIRECT) {
     [self switchModeToDirect:sender];
   } else if (new_mode != mozc::commands::DIRECT) {
@@ -344,11 +459,14 @@ CompositionMode GetCompositionMode(NSString *modeID) {
   }
 }
 
-- (void)switchDisplayMode:(id)sender {
+- (void)switchDisplayMode {
   if (gModeIdMap == NULL) {
     LOG(ERROR) << "gModeIdMap is not initialized correctly.";
     return;
   }
+  if (IsBannedApplication(gNoDisplayModeSwitchApps, *clientBundle_)) {
+    return;
+  }
 
   map<CompositionMode, NSString *>::const_iterator it = gModeIdMap->find(mode_);
   if (it == gModeIdMap->end()) {
@@ -356,28 +474,202 @@ CompositionMode GetCompositionMode(NSString *modeID) {
     return;
   }
 
-  [sender selectInputMode:it->second];
+  [[self client] selectInputMode:it->second];
 }
 
-- (void)handleInputMode {
-  if (!checkInputMode_) {
+- (void)commitText:(const char *)text client:(id)sender {
+  if (text == NULL) {
     return;
   }
 
-  // Get the config and set client-side behaviors
-  Config config;
-  if (!session_->GetConfig(&config)) {
-    LOG(ERROR) << "Cannot obtain the current config";
+  [sender insertText:[NSString stringWithUTF8String:text]
+    replacementRange:replacementRange_];
+  replacementRange_ = NSMakeRange(NSNotFound, 0);
+}
+
+- (void)launchWordRegisterTool:(id)client {
+  ::setenv(mozc::kWordRegisterEnvironmentName, "", 1);
+  if (!IsBannedApplication(gNoSelectedRangeApps, *clientBundle_)) {
+    NSRange selectedRange = [client selectedRange];
+    if (selectedRange.location != NSNotFound &&
+        selectedRange.length != NSNotFound &&
+        selectedRange.length > 0) {
+      NSString *text =
+        [[client attributedSubstringFromRange:selectedRange] string];
+      if (text != nil) {
+        :: setenv(mozc::kWordRegisterEnvironmentName, [text UTF8String], 1);
+      }
+    }
+  }
+  MacProcess::LaunchMozcTool("word_register_dialog");
+}
+
+- (void)invokeReconvert:(const SessionCommand *)command client:(id)sender {
+  if (IsBannedApplication(gNoSelectedRangeApps, *clientBundle_)) {
     return;
   }
 
-  InputMode input_mode = ASCII;
-  if (config.preedit_method() == Config::KANA) {
-    input_mode = KANA;
+  NSRange selectedRange = [sender selectedRange];
+  if (selectedRange.location == NSNotFound ||
+      selectedRange.length == NSNotFound) {
+    // the application does not support reconversion.
+    return;
+  }
+
+  DLOG(INFO) << selectedRange.location << ", " << selectedRange.length;
+  SessionCommand sending_command;
+  Output output;
+  sending_command.CopyFrom(*command);
+
+  if (selectedRange.length == 0) {
+    // Currently no range is selected for reconversion.  Tries to
+    // invoke UNDO instead.
+    [self invokeUndo:sender];
+    return;
+  }
+
+  NSAttributedString *text =
+      [sender attributedSubstringFromRange:selectedRange];
+  if (!sending_command.has_text()) {
+    sending_command.set_text([[text string] UTF8String]);
+  }
+
+  if (mozcClient_->SendCommand(sending_command, &output)) {
+    replacementRange_ = selectedRange;
+    [self processOutput:&output client:sender];
+  }
+}
+
+- (void)invokeUndo:(id)sender {
+  if (IsBannedApplication(gNoSelectedRangeApps, *clientBundle_)) {
+    return;
+  }
+
+  NSRange selectedRange = [sender selectedRange];
+  if (selectedRange.location == NSNotFound ||
+      selectedRange.length == NSNotFound ||
+      // Some applications such like iTunes does not return NSNotFound
+      // range but (0, 0).  However, the range starting with negative
+      // location has to be invalid, then we can reject such apps.
+      selectedRange.location == 0) {
+    return;
+  }
+
+  DLOG(INFO) << selectedRange.location << ", " << selectedRange.length;
+  SessionCommand command;
+  Output output;
+  command.set_type(SessionCommand::UNDO);
+  if (mozcClient_->SendCommand(command, &output)) {
+    [self processOutput:&output client:sender];
+  }
+}
+
+- (void)processOutput:(const mozc::commands::Output *)output client:(id)sender {
+  if (output == NULL) {
+    return;
+  }
+  if (!output->consumed()) {
+    return;
+  }
+
+  DLOG(INFO) << output->DebugString();
+  if (output->has_url()) {
+    NSString *url = [NSString stringWithUTF8String:output->url().c_str()];
+    [self openLink:[NSURL URLWithString:url]];
+  }
+
+  if (output->has_result()) {
+    [self commitText:output->result().value().c_str() client:sender];
+  }
+
+  // Handles deletion range.  We do not even handle it for some
+  // applications to prevent application crashes.
+  if (output->has_deletion_range() &&
+      !IsBannedApplication(gNoSelectedRangeApps, *clientBundle_)) {
+    if ([composedString_ length] == 0 &&
+        replacementRange_.location == NSNotFound) {
+      NSRange selectedRange = [sender selectedRange];
+      const mozc::commands::DeletionRange &deletion_range =
+          output->deletion_range();
+      if (selectedRange.location != NSNotFound ||
+          selectedRange.length != NSNotFound ||
+          selectedRange.location + deletion_range.offset() > 0) {
+        // The offset is a negative value.  See session/commands.proto for
+        // the details.
+        selectedRange.location += deletion_range.offset();
+        selectedRange.length += deletion_range.length();
+        replacementRange_ = selectedRange;
+      }
+    } else {
+      // We have to consider the case that there is already
+      // composition and/or we already set the position of the
+      // composition by replacementRange_.  We do nothing here at this
+      // time because we already found that it will involve several
+      // buggy behaviors with Carbon apps and MS Office.
+      // TODO(mukai): find the right behavior.
+    }
+  }
+
+  [self updateComposedString:&(output->preedit())];
+  [self updateCandidates:output];
+
+  if (output->has_mode()) {
+    CompositionMode new_mode = output->mode();
+    // Do not allow HALF_ASCII with empty composition.  This should be
+    // handled in the converter, but just in case.
+    if (new_mode == mozc::commands::HALF_ASCII &&
+        (!output->has_preedit() || output->preedit().segment_size() == 0)) {
+      new_mode = mozc::commands::DIRECT;
+      [self switchMode:new_mode client:sender];
+    }
+    if (new_mode != mode_) {
+      mode_ = new_mode;
+      [self switchDisplayMode];
+    }
+  }
+
+  if (output->has_launch_tool_mode()) {
+    switch (output->launch_tool_mode()) {
+      case mozc::commands::Output::CONFIG_DIALOG:
+        MacProcess::LaunchMozcTool("config_dialog");
+        break;
+      case mozc::commands::Output::DICTIONARY_TOOL:
+        MacProcess::LaunchMozcTool("dictionary_tool");
+        break;
+      case mozc::commands::Output::WORD_REGISTER_DIALOG:
+        [self launchWordRegisterTool:sender];
+        break;
+      default:
+        // do nothing
+        break;
+    }
+  }
+
+  // Handle callbacks.
+  if (output->has_callback() && output->callback().has_session_command()) {
+    if (output->callback().has_delay_millisec()) {
+      callback_command_.CopyFrom(output->callback());
+      // In the current implementation, if the subsequent key event also makes
+      // callback, the second callback will be called in the timimg of the first
+      // callback.
+      [self performSelector:@selector(sendCallbackCommand)
+                 withObject:nil
+                 afterDelay:output->callback().has_delay_millisec() / 1000.0];
+      return;
+    }
+    const SessionCommand &callback_command =
+        output->callback().session_command();
+    if (callback_command.type() == SessionCommand::CONVERT_REVERSE) {
+      [self invokeReconvert:&callback_command client:sender];
+    } else if (callback_command.type() == SessionCommand::UNDO) {
+      [self invokeUndo:sender];
+    } else {
+      Output output_for_callback;
+      if (mozcClient_->SendCommand(callback_command, &output_for_callback)) {
+        [self processOutput:&output_for_callback client:sender];
+      }
+    }
   }
-  [keyCodeMap_ setInputMode:input_mode];
-  yenSignCharacter_ = config.yen_sign_character();
-  checkInputMode_ = NO;
 }
 
 #pragma mark Mozc Server methods
@@ -399,6 +691,13 @@ CompositionMode GetCompositionMode(NSString *modeID) {
 }
 
 - (void)updateComposedString:(const Preedit *)preedit {
+  // If the last and the current composed string length is 0,
+  // we don't call updateComposition.
+  if (([composedString_ length] == 0) &&
+      ((preedit == NULL || preedit->segment_size() == 0))) {
+    return;
+  }
+
   [composedString_
     deleteCharactersInRange:NSMakeRange(0, [composedString_ length])];
   cursorPosition_ = NSNotFound;
@@ -425,6 +724,7 @@ CompositionMode GetCompositionMode(NSString *modeID) {
   }
   if ([composedString_ length] == 0) {
     [originalString_ setString:@""];
+    replacementRange_ = NSMakeRange(NSNotFound, 0);
   }
 
   // Make composed string visible to the client applications.
@@ -436,13 +736,12 @@ CompositionMode GetCompositionMode(NSString *modeID) {
     DLOG(INFO) << "Nothing is committed.";
     return;
   }
-  [sender insertText:[composedString_ string]
-          replacementRange:NSMakeRange(NSNotFound, 0)];
+  [self commitText:[[composedString_ string] UTF8String] client:sender];
 
   SessionCommand command;
   Output output;
   command.set_type(SessionCommand::SUBMIT);
-  session_->SendCommand(command, &output);
+  mozcClient_->SendCommand(command, &output);
   [self clearCandidates];
   [self updateComposedString:NULL];
 }
@@ -468,15 +767,7 @@ CompositionMode GetCompositionMode(NSString *modeID) {
       NSMakeRange(cursorPosition_, 0);
 }
 
-- (void)updateCandidates:(const Output *)output client:(id)sender {
-  if (output == NULL) {
-    [self clearCandidates];
-    return;
-  }
-
-  rendererCommand_->set_type(RendererCommand::UPDATE);
-  rendererCommand_->mutable_output()->CopyFrom(*output);
-
+- (void)delayedUpdateCandidates {
   // The candidate window position is not recalculated if the
   // candidate already appears on the screen.  Therefore, if a user
   // moves client application window by mouse, candidate window won't
@@ -485,10 +776,26 @@ CompositionMode GetCompositionMode(NSString *modeID) {
   //    cursor position correctly.  The candidate window moves
   //    frequently with those application, which irritates users.
   //  - Kotoeri does this too.
-  if (!rendererCommand_->visible()) {
+  if ((!rendererCommand_->visible()) &&
+      (rendererCommand_->output().candidates().candidate_size() > 0)) {
     NSRect preeditRect = NSZeroRect;
-    [sender attributesForCharacterIndex:output->candidates().position()
-            lineHeightRectangle:&preeditRect];
+    const int32 position = rendererCommand_->output().candidates().position();
+    // Some applications throws error when we call attributesForCharacterIndex.
+    DLOG(INFO) << "attributesForCharacterIndex: " << position;
+    @try {
+      [[self client] attributesForCharacterIndex:position
+                             lineHeightRectangle:&preeditRect];
+    }
+    @catch (NSException *exception) {
+      LOG(ERROR) << "Exception from [" << *clientBundle_ << "] "
+                 << [[exception name] UTF8String] << ","
+                 << [[exception reason] UTF8String];
+    }
+    DLOG(INFO) << "  preeditRect: (("
+               << preeditRect.origin.x << ", "
+               << preeditRect.origin.y << "), ("
+               << preeditRect.size.width << ", "
+               << preeditRect.size.height << "))";
     NSScreen *baseScreen = nil;
     NSRect baseFrame = NSZeroRect;
     for (baseScreen in [NSScreen screens]) {
@@ -508,24 +815,87 @@ CompositionMode GetCompositionMode(NSString *modeID) {
         baseHeight - preeditRect.origin.y);
   }
 
-  rendererCommand_->set_visible(output->candidates().candidate_size() > 0);
+  rendererCommand_->set_visible(
+    rendererCommand_->output().candidates().candidate_size() > 0);
   if (candidateController_) {
     candidateController_->ExecCommand(*rendererCommand_);
   }
 }
 
+- (void)updateCandidates:(const Output *)output {
+  if (output == NULL) {
+    [self clearCandidates];
+    return;
+  }
+
+  rendererCommand_->set_type(RendererCommand::UPDATE);
+  rendererCommand_->mutable_output()->CopyFrom(*output);
+
+  // Runs delayedUpdateCandidates in the next event loop.
+  // This is because some applications like Google Docs with Chrome returns
+  // incorrect cursor position if we call attributesForCharacterIndex here.
+  [self performSelector:@selector(delayedUpdateCandidates)
+             withObject:nil
+             afterDelay:0];
+}
+
 - (void)openLink:(NSURL *)url {
   // Open a link specified by |url|.  Any opening link behavior should
   // call this method because it checks the capability of application.
   // On some application like login window of screensaver, opening
   // link behavior should not happen because it can cause some
   // security issues.
-  if (!clientBundle_ || [gNoOpenLinkApps containsObject:clientBundle_]) {
+  if (!clientBundle_ || IsBannedApplication(gNoOpenLinkApps, *clientBundle_)) {
     return;
   }
   [[NSWorkspace sharedWorkspace] openURL:url];
 }
 
+- (BOOL)fillSurroundingContext:(mozc::commands::Context *)context
+                        client:(id<IMKTextInput>)client {
+  NSRange selectedRange;
+  if ([composedString_ length] == 0) {
+    selectedRange = [client selectedRange];
+  } else {
+    // When composed string exists we calls markedRange to get the surrounding
+    // text of the composed string.
+    selectedRange = [client markedRange];
+  }
+  if (selectedRange.location == NSNotFound ||
+      selectedRange.length == NSNotFound) {
+    return false;
+  }
+  NSInteger totalLength = [client length];
+  if (totalLength == 0 || totalLength == NSNotFound) {
+    return false;
+  }
+  NSRange precedingRange = NSMakeRange(0, selectedRange.location);
+  if (selectedRange.location > kMaxSurroundingLength) {
+    precedingRange =
+        NSMakeRange(selectedRange.location - kMaxSurroundingLength,
+                    kMaxSurroundingLength);
+  }
+  NSRange followingRange =
+      NSMakeRange(selectedRange.location + selectedRange.length,
+                  totalLength - selectedRange.location - selectedRange.length);
+  if (followingRange.length > kMaxSurroundingLength) {
+    followingRange.length = kMaxSurroundingLength;
+  }
+  NSString *precedingString =
+    [[client attributedSubstringFromRange:precedingRange] string];
+  NSString *followingString =
+    [[client attributedSubstringFromRange:followingRange] string];
+  if (precedingString) {
+    context->set_preceding_text([precedingString UTF8String]);
+    DLOG(INFO) << "preceding_text: \"" << context->preceding_text() << "\"";
+  }
+  if (followingString) {
+    context->set_following_text([followingString UTF8String]);
+    DLOG(INFO) << "following_text: \"" << context->following_text() << "\"";
+  }
+  return true;
+}
+
 - (BOOL)handleEvent:(NSEvent *)event client:(id)sender {
   if ([event type] == NSCursorUpdate) {
     [self updateComposition];
@@ -534,6 +904,46 @@ CompositionMode GetCompositionMode(NSString *modeID) {
   if ([event type] != NSKeyDown && [event type] != NSFlagsChanged) {
     return NO;
   }
+  // Cancels the callback.
+  callback_command_.Clear();
+
+  // Handle KANA key and EISU key.  We explicitly handles this here
+  // for mode switch because some text area such like iPhoto person
+  // name editor does not call setValue:forTag:client: method.
+  // see: http://www.google.com/support/forum/p/ime/thread?tid=3aafb74ff71a1a69&hl=ja&fid=3aafb74ff71a1a690004aa3383bc9f5d
+  if ([event type] == NSKeyDown) {
+    NSTimeInterval currentTime = [[NSDate date] timeIntervalSince1970];
+    const NSTimeInterval elapsedTime = currentTime - lastKeyDownTime_;
+    const bool isDoubleTap = ([event keyCode] == lastKeyCode_) &&
+                             (elapsedTime < kDoubleTapInterval);
+    lastKeyDownTime_ = currentTime;
+    lastKeyCode_ = [event keyCode];
+
+    // these calling of switchMode: can be duplicated if the
+    // application sends the setValue:forTag:client: and handleEvent:
+    // at the same key event, but that's okay because switchMode:
+    // method does nothing if the new mode is same as the current
+    // mode.
+    if ([event keyCode] == kVK_JIS_Kana) {
+      [self switchMode:mozc::commands::HIRAGANA client:sender];
+      [self switchDisplayMode];
+      if (isDoubleTap) {
+        SessionCommand command;
+        command.set_type(SessionCommand::CONVERT_REVERSE);
+        [self invokeReconvert:&command client:sender];
+      }
+    } else if ([event keyCode] == kVK_JIS_Eisu) {
+      if (isDoubleTap) {
+        SessionCommand command;
+        command.set_type(SessionCommand::COMMIT_RAW_TEXT);
+        [self sendCommand:command];
+      }
+      CompositionMode new_mode = ([composedString_ length] == 0) ?
+          mozc::commands::DIRECT : mozc::commands::HALF_ASCII;
+      [self switchMode:new_mode client:sender];
+      [self switchDisplayMode];
+    }
+  }
 
   if ([keyCodeMap_ isModeSwitchingKey:event]) {
     // Special hack for Eisu/Kana keys.  Sometimes those key events
@@ -542,8 +952,6 @@ CompositionMode GetCompositionMode(NSString *modeID) {
     return YES;
   }
 
-  [self handleInputMode];
-
   // Get the Mozc key event
   KeyEvent keyEvent;
   if (![keyCodeMap_ getMozcKeyCodeFromKeyEvent:event
@@ -556,20 +964,19 @@ CompositionMode GetCompositionMode(NSString *modeID) {
   // If the key event is turn on event, the key event has to be sent
   // to the server anyway.
   if (mode_ == mozc::commands::DIRECT &&
-      !ImeSwitchUtil::IsTurnOnInDirectMode(keyEvent)) {
+      !ImeSwitchUtil::IsDirectModeCommand(keyEvent)) {
     // Yen sign special hack: although the current mode is DIRECT,
     // backslash is sent instead of yen sign for JIS yen key with no
     // modifiers.  This behavior is based on the configuration.
     if ([event keyCode] == kVK_JIS_Yen &&
         [event modifierFlags] == 0 &&
         yenSignCharacter_ == mozc::config::Config::BACKSLASH) {
-      [sender insertText:@"\\" replacementRange:NSMakeRange(NSNotFound, 0)];
+      [self commitText:"\\" client:sender];
       return YES;
     }
     return NO;
   }
 
-
   // Send the key event to the server actually
   Output output;
 
@@ -577,83 +984,76 @@ CompositionMode GetCompositionMode(NSString *modeID) {
     [originalString_ appendFormat:@"%c", keyEvent.key_code()];
   }
 
-  if (!session_->SendKey(keyEvent, &output)) {
+  mozc::commands::Context context;
+  if (suppressSuggestion_) {
+    // TODO(komatsu, horo): Support Google Omnibox too.
+    context.add_experimental_features("google_search_box");
+  }
+  keyEvent.set_mode(mode_);
+
+  if (!IsBannedApplication(gNoSelectedRangeApps, *clientBundle_)) {
+    [self fillSurroundingContext:&context client:sender];
+  }
+  if (!mozcClient_->SendKeyWithContext(keyEvent, context, &output)) {
     return NO;
   }
 
-  DLOG(INFO) << output.DebugString();
-  if (output.has_url()) {
-    NSString *url = [NSString stringWithUTF8String:output.url().c_str()];
-    [self openLink:[NSURL URLWithString:url]];
-    output.clear_url();
-  }
-
-  if (output.has_result()) {
-    NSString *resultText =
-        [NSString
-          stringWithUTF8String:output.result().value().c_str()];
-    [sender insertText:resultText
-      replacementRange:NSMakeRange(NSNotFound, 0)];
-  }
-  if (output.consumed()) {
-    [self updateComposedString:&(output.preedit())];
-    [self updateCandidates:&output client:sender];
-  }
-  if (output.has_mode()) {
-    CompositionMode new_mode = output.mode();
-    // Do not allow HALF_ASCII with empty composition.  This should be
-    // handled in the converter, but just in case.
-    if (new_mode == mozc::commands::HALF_ASCII &&
-        (!output.has_preedit() || output.preedit().segment_size() == 0)) {
-      new_mode = mozc::commands::DIRECT;
-      [self switchMode:new_mode client:sender];
-    }
-    if (new_mode != mode_) {
-      mode_ = new_mode;
-      [self switchDisplayMode:sender];
-    }
-  }
-
+  [self processOutput:&output client:sender];
   return output.consumed();
 }
 
+- (void)sendCallbackCommand {
+  if (callback_command_.has_session_command()) {
+    const SessionCommand command = callback_command_.session_command();
+    callback_command_.Clear();
+    [self sendCommand:command];
+  }
+}
+
 #pragma mark callbacks
-- (void)candidateClicked:(int)id {
-  SessionCommand command;
-  command.set_type(SessionCommand::SELECT_CANDIDATE);
-  command.set_id(id);
+- (void)sendCommand:(const SessionCommand &)command {
   Output output;
-  if (!session_->SendCommand(command, &output)) {
+  if (!mozcClient_->SendCommand(command, &output)) {
     return;
   }
 
-  [self updateComposedString:&(output.preedit())];
-  [self updateCandidates:&output client:[self client]];
+  [self processOutput:&output client:[self client]];
+}
+
+- (IBAction)reconversionClicked:(id)sender {
+  SessionCommand command;
+  command.set_type(SessionCommand::CONVERT_REVERSE);
+  [self invokeReconvert:&command client:[self client]];
 }
 
 - (IBAction)configClicked:(id)sender {
-  mozc::MacProcess::LaunchMozcTool("config_dialog");
+  MacProcess::LaunchMozcTool("config_dialog");
 }
 
 - (IBAction)dictionaryToolClicked:(id)sender {
-  mozc::MacProcess::LaunchMozcTool("dictionary_tool");
+  MacProcess::LaunchMozcTool("dictionary_tool");
 }
 
-- (IBAction)characterPadClicked:(id)sender {
-  mozc::MacProcess::LaunchMozcTool("character_pad");
+- (IBAction)registerWordClicked:(id)sender {
+  [self launchWordRegisterTool:[self client]];
+}
+
+- (IBAction)characterPaletteClicked:(id)sender {
+  MacProcess::LaunchMozcTool("character_palette");
+}
+
+- (IBAction)handWritingClicked:(id)sender {
+  MacProcess::LaunchMozcTool("hand_writing");
 }
 
 - (IBAction)aboutDialogClicked:(id)sender {
-  mozc::MacProcess::LaunchMozcTool("about_dialog");
+  MacProcess::LaunchMozcTool("about_dialog");
 }
 
 - (void)outputResult:(mozc::commands::Output *)output {
   if (output == NULL || !output->has_result()) {
     return;
   }
-  NSString *resultText =
-      [NSString stringWithUTF8String:output->result().value().c_str()];
-  [[self client] insertText:resultText
-           replacementRange:NSMakeRange(NSNotFound, 0)];
+  [self commitText:output->result().value().c_str() client:[self client]];
 }
 @end
