@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -36,8 +36,11 @@
 
 #include "base/base.h"
 #include "base/const.h"
+#include "base/crash_report_handler.h"
+#include "base/logging.h"
 #include "base/run_level.h"
-#include "client/session.h"
+#include "client/client.h"
+#include "config/stats_config_util.h"
 
 int main(int argc, char *argv[]) {
   if (!mozc::RunLevel::IsValidClientRunLevel()) {
@@ -45,7 +48,13 @@ int main(int argc, char *argv[]) {
   }
 
   NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
-  InitGoogleWithBreakPad(argv[0], &argc, &argv, false);
+#ifdef GOOGLE_JAPANESE_INPUT_BUILD
+  if (mozc::config::StatsConfigUtil::IsEnabled()) {
+    mozc::CrashReportHandler::Initialize(false);
+  }
+#endif
+  InitGoogle(argv[0], &argc, &argv, false);
+
   IMKServer *imkServer = [GoogleJapaneseInputServer getServer];
   if (!imkServer) {
     LOG(FATAL) << mozc::kProductNameInEnglish << " failed to initialize";
@@ -58,10 +67,10 @@ int main(int argc, char *argv[]) {
   // Start the converter server at this time explicitly to prevent the
   // slow-down of the response for initial key event.
   {
-    scoped_ptr<mozc::client::Session> session(new mozc::client::Session);
-    session->PingServer();
+    scoped_ptr<mozc::client::Client> client(new mozc::client::Client);
+    client->PingServer();
   }
-  RunApplicationEventLoop();
+  NSApplicationMain(argc, (const char **)argv);
   [pool drain];
 #ifdef GOOGLE_JAPANESE_INPUT_BUILD
   mozc::CrashReportHandler::Uninitialize();
