@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,13 +29,21 @@
 
 // Classes managing dictionary entries.
 
-#include <algorithm>
+#include "rewriter/dictionary_generator.h"
 
+#include <algorithm>
+#include <map>
+#include <string>
+#include <vector>
+
+#include "base/base.h"
 #include "base/file_stream.h"
 #include "base/freelist.h"
+#include "base/logging.h"
 #include "base/util.h"
+#include "data_manager/user_pos_manager.h"
+#include "dictionary/pos_matcher.h"
 #include "dictionary/user_pos.h"
-#include "rewriter/dictionary_generator.h"
 
 namespace mozc {
 namespace rewriter {
@@ -74,7 +82,13 @@ static const size_t kTokenSize = 1000;
 
 DictionaryGenerator::DictionaryGenerator()
     : token_pool_(new ObjectPool<Token>(kTokenSize)),
-      token_map_(new map<uint64, Token *>) {}
+      token_map_(new map<uint64, Token *>),
+      user_pos_(
+          new UserPOS(UserPosManager::GetUserPosManager()->GetUserPOSData())),
+      open_bracket_id_(UserPosManager::GetUserPosManager()->GetPOSMatcher()
+                       ->GetOpenBracketId()),
+      close_bracket_id_(UserPosManager::GetUserPosManager()->GetPOSMatcher()
+                        ->GetCloseBracketId()) {}
 
 DictionaryGenerator::~DictionaryGenerator() {}
 
@@ -144,7 +158,13 @@ bool DictionaryGenerator::Output(const string &filename) const {
     const uint32 cost = 10 * num_same_keys;
 
     uint16 id;
-    CHECK(UserPOS::GetPOSIDs(pos, &id)) << "Unknown POS type: " << pos;
+    if (pos == "\xE6\x8B\xAC\xE5\xBC\xA7\xE9\x96\x8B") {      // "括弧開"
+      id = open_bracket_id_;
+    } else if (pos == "\xE6\x8B\xAC\xE5\xBC\xA7\xE9\x96\x89") {  // "括弧閉"
+      id = close_bracket_id_;
+    } else {
+      CHECK(user_pos_->GetPOSIDs(pos, &id)) << "Unknown POS type: " << pos;
+    }
 
     // Output in mozc dictionary format
     ofs << token.key() << "\t"
