@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -30,50 +30,70 @@
 #include "rewriter/user_segment_history_rewriter.h"
 
 #include <algorithm>
+#include <cctype>
 #include <set>
 #include <string>
 #include <vector>
 
+#include "base/base.h"
+#include "base/compiler_specific.h"
 #include "base/config_file_stream.h"
+#include "base/file_util.h"
+#include "base/logging.h"
+#include "base/number_util.h"
+#include "base/string_piece.h"
 #include "base/util.h"
-#include "transliteration/transliteration.h"
-#include "converter/character_form_manager.h"
-#include "converter/pos_matcher.h"
+#include "config/character_form_manager.h"
+#include "config/config.pb.h"
+#include "config/config_handler.h"
+#include "converter/conversion_request.h"
 #include "converter/segments.h"
-#include "storage/lru_storage.h"
+#include "dictionary/pos_group.h"
+#include "dictionary/pos_matcher.h"
 #include "rewriter/rewriter_interface.h"
-#include "session/config_handler.h"
-#include "session/config.pb.h"
+#include "rewriter/variants_rewriter.h"
+#include "storage/lru_storage.h"
+#include "transliteration/transliteration.h"
 #include "usage_stats/usage_stats.h"
 
 namespace mozc {
 
+using config::CharacterFormManager;
+using config::Config;
+using storage::LRUStorage;
+
 namespace {
 const uint32 kValueSize = 4;
 const uint32 kLRUSize   = 20000;
 const uint32 kSeedValue = 0xf28defe3;
 const uint32 kMaxCandidatesSize = 255;
 
+const char kFileName[] = "user://segment.db";
+
+LRUStorage *g_lru_storage = NULL;
+
+// Temporarily disable unused private field warning against
+// FeatureValue::reserved_ from Clang.
+// We use MOZC_CLANG_HAS_WARNING to check whether "-Wunused-private-field" is
+// available, because XCode 4.4 clang (based on LLVM 3.1svn) doesn't have it.
+MOZC_CLANG_PUSH_WARNING();
+#if MOZC_CLANG_HAS_WARNING(unused-private-field)
+MOZC_CLANG_DISABLE_WARNING(unused-private-field);
+#endif
 class FeatureValue {
  public:
   FeatureValue() : feature_type_(1), reserved_(0) {}
   bool IsValid() const {
     return (feature_type_ == 1);
   }
+
  private:
   uint32 feature_type_ : 1;   // always 1
   uint32 reserved_     : 31;  // this area is reserved for future
 };
+MOZC_CLANG_POP_WARNING();
 
-// define kLidGroup[]
-#include "rewriter/user_segment_history_rewriter_rule.h"
-
-// return the POS group of the given candidate
-uint16 GetPosGroup(const Segment::Candidate &candidate) {
-  return kLidGroup[candidate.lid];
-}
-
-bool IsPunctuation(const string &str) {
+bool IsPunctuationInternal(const string &str) {
   // return (str == "。" || str == "｡" ||
   // str == "、" || str == "､" ||
   // str == "，" || str == "," ||
@@ -84,13 +104,14 @@ bool IsPunctuation(const string &str) {
           str == "\xEF\xBC\x8E"  || str == ".");
 }
 
-bool IsPunctuation(const Segment &seg,
-                   const Segment::Candidate &candidate) {
-  return (POSMatcher::IsJapanesePunctuations(candidate.lid) &&
-          candidate.lid == candidate.rid &&
-          IsPunctuation(seg.key()) && IsPunctuation(candidate.value));
-}
-
+// Temporarily disable unused private field warning against
+// KeyTriggerValue::reserved_ from Clang.
+// We use MOZC_CLANG_HAS_WARNING to check whether "-Wunused-private-field" is
+// available, because XCode 4.4 clang (based on LLVM 3.1svn) doesn't have it.
+MOZC_CLANG_PUSH_WARNING();
+#if MOZC_CLANG_HAS_WARNING(unused-private-field)
+MOZC_CLANG_DISABLE_WARNING(unused-private-field);
+#endif
 class KeyTriggerValue {
  public:
   KeyTriggerValue()
@@ -114,17 +135,12 @@ class KeyTriggerValue {
   // want to encode POS, freq etc.
   uint32 candidates_size_ : 8;   // candidate size
 };
-
-struct ScoreType {
-  uint32 last_access_time;
-  uint32 score;
-  const Segment::Candidate *candidate;
-};
+MOZC_CLANG_POP_WARNING();
 
 class ScoreTypeCompare {
  public:
-  bool operator() (const ScoreType &a,
-                   const ScoreType &b) const {
+  bool operator() (const UserSegmentHistoryRewriter::ScoreType &a,
+                   const UserSegmentHistoryRewriter::ScoreType &b) const {
     if (a.score != b.score) {
       return (a.score > b.score);
     }
@@ -142,10 +158,59 @@ inline int GetDefaultCandidateIndex(const Segment &segment) {
     }
   }
 
-  LOG(WARNING) << "Cannot find default candidate";
+  LOG(WARNING) << "Cannot find default candidate. "
+               << "key: " << segment.key() << ", "
+               << "candidates_size: " << segment.candidates_size();
   return 0;
 }
 
+// JoinStringWithTabN joins N strings with TAB delimiters ('\t') in a way
+// similar to Util::JoinStrings() and/or Util::AppendStringWithDelimiter() but
+// in a more efficient way. Since this module is called every key stroke and
+// performs many string concatenation, we use these functions instead of ones
+// from Util.
+inline void JoinStringsWithTab2(
+    const StringPiece s1, const StringPiece s2, string *output) {
+  // Pre-allocate the buffer, including 1 TAB delimiter.
+  output->reserve(s1.size() + s2.size() + 1);
+  output->assign(s1.data(), s1.size()).append("\t")
+      .append(s2.data(), s2.size());
+}
+
+inline void JoinStringsWithTab3(
+    const StringPiece s1, const StringPiece s2, const StringPiece s3,
+    string *output) {
+  // Pre-allocate the buffer, including 2 TAB delimiters.
+  output->reserve(s1.size() + s2.size() + s3.size() + 2);
+  output->assign(s1.data(), s1.size()).append("\t")
+      .append(s2.data(), s2.size()).append("\t")
+      .append(s3.data(), s3.size());
+}
+
+inline void JoinStringsWithTab4(
+    const StringPiece s1, const StringPiece s2, const StringPiece s3,
+    const StringPiece s4, string *output) {
+  // Pre-allocate the buffer, including 3 TAB delimiters.
+  output->reserve(s1.size() + s2.size() + s3.size() + s4.size() + 3);
+  output->assign(s1.data(), s1.size()).append("\t")
+      .append(s2.data(), s2.size()).append("\t")
+      .append(s3.data(), s3.size()).append("\t")
+      .append(s4.data(), s4.size());
+}
+
+inline void JoinStringsWithTab5(
+    const StringPiece s1, const StringPiece s2, const StringPiece s3,
+    const StringPiece s4, const StringPiece s5, string *output) {
+  // Pre-allocate the buffer, including 4 TAB delimiters.
+  output->reserve(
+      s1.size() + s2.size() + s3.size() + s4.size() + s5.size() + 4);
+  output->assign(s1.data(), s1.size()).append("\t")
+      .append(s2.data(), s2.size()).append("\t")
+      .append(s3.data(), s3.size()).append("\t")
+      .append(s4.data(), s4.size()).append("\t")
+      .append(s5.data(), s5.size());
+}
+
 // Feature "Left Right"
 inline bool GetFeatureLR(const Segments &segments, size_t i,
                          const string &base_key,
@@ -156,10 +221,12 @@ inline bool GetFeatureLR(const Segments &segments, size_t i,
   }
   const int j1 = GetDefaultCandidateIndex(segments.segment(i - 1));
   const int j2 = GetDefaultCandidateIndex(segments.segment(i + 1));
-  *value = string("LR") + '\t' + base_key + '\t' +
-      segments.segment(i - 1).candidate(j1).value + '\t' +
-      base_value + '\t' +
-      segments.segment(i + 1).candidate(j2).value;
+  JoinStringsWithTab5(StringPiece("LR", 2),
+                      base_key,
+                      segments.segment(i - 1).candidate(j1).value,
+                      base_value,
+                      segments.segment(i + 1).candidate(j2).value,
+                      value);
   return true;
 }
 
@@ -173,10 +240,12 @@ inline bool GetFeatureLL(const Segments &segments, size_t i,
   }
   const int j1 = GetDefaultCandidateIndex(segments.segment(i - 2));
   const int j2 = GetDefaultCandidateIndex(segments.segment(i - 1));
-  *value = string("LL") + '\t' + base_key + '\t' +
-      segments.segment(i - 2).candidate(j1).value + '\t' +
-      segments.segment(i - 1).candidate(j2).value + '\t' +
-      base_value;
+  JoinStringsWithTab5(StringPiece("LL", 2),
+                      base_key,
+                      segments.segment(i - 2).candidate(j1).value,
+                      segments.segment(i - 1).candidate(j2).value,
+                      base_value,
+                      value);
   return true;
 }
 
@@ -190,10 +259,12 @@ inline bool GetFeatureRR(const Segments &segments, size_t i,
   }
   const int j1 = GetDefaultCandidateIndex(segments.segment(i + 1));
   const int j2 = GetDefaultCandidateIndex(segments.segment(i + 2));
-  *value = string("RR") + '\t' + base_key + '\t' +
-      base_value + '\t' +
-      segments.segment(i + 1).candidate(j1).value + '\t' +
-      segments.segment(i + 2).candidate(j2).value;
+  JoinStringsWithTab5(StringPiece("RR", 2),
+                      base_key,
+                      base_value,
+                      segments.segment(i + 1).candidate(j1).value,
+                      segments.segment(i + 2).candidate(j2).value,
+                      value);
   return true;
 }
 
@@ -206,9 +277,11 @@ inline bool GetFeatureL(const Segments &segments, size_t i,
     return false;
   }
   const int j = GetDefaultCandidateIndex(segments.segment(i - 1));
-  *value = string("L") + '\t' + base_key + '\t' +
-      segments.segment(i - 1).candidate(j).value + '\t' +
-      base_value;
+  JoinStringsWithTab4(StringPiece("L", 1),
+                      base_key,
+                      segments.segment(i - 1).candidate(j).value,
+                      base_value,
+                      value);
   return true;
 }
 
@@ -221,56 +294,20 @@ inline bool GetFeatureR(const Segments &segments, size_t i,
     return false;
   }
   const int j = GetDefaultCandidateIndex(segments.segment(i + 1));
-  *value = string("R") + '\t' + base_key + '\t' +
-      base_value + '\t' +
-      segments.segment(i + 1).candidate(j).value;
+  JoinStringsWithTab4(StringPiece("R", 1),
+                      base_key,
+                      base_value,
+                      segments.segment(i + 1).candidate(j).value,
+                      value);
   return true;
 }
 
-// Feature "Left Number"
-inline bool GetFeatureLN(const Segments &segments, size_t i,
-                         const string &base_key,
-                         const string &base_value, string *value) {
-  DCHECK(value);
-  if (i < 1) {
-    return false;
-  }
-  const int j = GetDefaultCandidateIndex(segments.segment(i - 1));
-  const Segment::Candidate &candidate = segments.segment(i - 1).candidate(j);
-  if (POSMatcher::IsNumber(candidate.rid) ||
-      POSMatcher::IsKanjiNumber(candidate.rid) ||
-      Util::GetScriptType(candidate.value) == Util::NUMBER) {
-    *value = string("LN") + '\t' + base_key + '\t' + base_value;
-    return true;
-  }
-  return false;
-}
-
-// Feature "Right Number"
-inline bool GetFeatureRN(const Segments &segments, size_t i,
-                         const string &base_key,
-                         const string &base_value, string *value) {
-  DCHECK(value);
-  if (i + 1 >= segments.segments_size()) {
-    return false;
-  }
-  const int j = GetDefaultCandidateIndex(segments.segment(i + 1));
-  const Segment::Candidate &candidate = segments.segment(i + 1).candidate(j);
-  if (POSMatcher::IsNumber(candidate.lid) ||
-      POSMatcher::IsKanjiNumber(candidate.lid) ||
-      Util::GetScriptType(candidate.value) == Util::NUMBER) {
-    *value = string("RN") + '\t' + base_key + '\t' + base_value;
-    return true;
-  }
-  return false;
-}
-
 // Feature "Current"
 inline bool GetFeatureC(const Segments &segments, size_t i,
                         const string &base_key,
                         const string &base_value, string *value) {
   DCHECK(value);
-  *value = string("C") + '\t' + base_key + '\t' + base_value;
+  JoinStringsWithTab3(StringPiece("C", 1), base_key, base_value, value);
   return true;
 }
 
@@ -282,7 +319,7 @@ inline bool GetFeatureS(const Segments &segments, size_t i,
   if (segments.segments_size() - segments.history_segments_size() != 1) {
     return false;
   }
-  *value = string("S") + '\t' + base_key + '\t' + base_value;
+  JoinStringsWithTab3(StringPiece("S", 1), base_key, base_value, value);
   return true;
 }
 
@@ -290,7 +327,7 @@ inline bool GetFeatureS(const Segments &segments, size_t i,
 // used for number rewrite
 inline bool GetFeatureN(uint16 type, string *value) {
   DCHECK(value);
-  *value = string("N") + '\t' + Util::SimpleItoa(type);
+  JoinStringsWithTab2(StringPiece("N", 1), NumberUtil::SimpleItoa(type), value);
   return true;
 }
 
@@ -309,7 +346,7 @@ bool IsNumberSegment(const Segment &seg) {
 }
 
 void GetValueByType(const Segment *segment,
-                    Segment::Candidate::Style style,
+                    NumberUtil::NumberString::Style style,
                     string *output) {
   DCHECK(output);
   for (size_t i = 0; i < segment->candidates_size(); ++i) {
@@ -334,29 +371,31 @@ void NormalizeCandidate(const Segment *segment, int n,
 
   string result = candidate.value;
   switch (candidate.style) {
-    case Segment::Candidate::DEFAULT_STYLE:
+    case NumberUtil::NumberString::DEFAULT_STYLE:
       CharacterFormManager::GetCharacterFormManager()->
           ConvertConversionString(candidate.value, &result);
       break;
-    case Segment::Candidate::NUMBER_SEPARATED_ARABIC_HALFWIDTH:
-    case Segment::Candidate::NUMBER_SEPARATED_ARABIC_FULLWIDTH:
+    case NumberUtil::NumberString::NUMBER_SEPARATED_ARABIC_HALFWIDTH:
+    case NumberUtil::NumberString::NUMBER_SEPARATED_ARABIC_FULLWIDTH:
       // Convert separated arabic here and don't use character form manager
       // so that suppressing mixed form of candidates
       // ("1，234" etc.)
       // and the forms of separated arabics are learned in converter using
       // style.
       {
-        const config::Config::CharacterForm
-            form = CharacterFormManager::GetCharacterFormManager()->
+        const Config::CharacterForm form =
+            CharacterFormManager::GetCharacterFormManager()->
             GetConversionCharacterForm("0");
-        if (form == config::Config::FULL_WIDTH) {
-          GetValueByType(segment,
-                         Segment::Candidate::NUMBER_SEPARATED_ARABIC_FULLWIDTH,
-                         &result);
-        } else if (form == config::Config::HALF_WIDTH) {
-          GetValueByType(segment,
-                         Segment::Candidate::NUMBER_SEPARATED_ARABIC_HALFWIDTH,
-                         &result);
+        if (form == Config::FULL_WIDTH) {
+          GetValueByType(
+              segment,
+              NumberUtil::NumberString::NUMBER_SEPARATED_ARABIC_FULLWIDTH,
+              &result);
+        } else if (form == Config::HALF_WIDTH) {
+          GetValueByType(
+              segment,
+              NumberUtil::NumberString::NUMBER_SEPARATED_ARABIC_HALFWIDTH,
+              &result);
         }
       }
       break;
@@ -389,7 +428,14 @@ bool GetSameValueCandidatePosition(const Segment *segment,
   return false;
 }
 
-bool SortCandidates(const vector<ScoreType> &sorted_scores, Segment *segment) {
+bool IsT13NCandidate(const Segment::Candidate &cand) {
+  // Regard the cand with 0-id as the transliterated candidate.
+  return (cand.lid == 0 && cand.rid == 0);
+}
+}  // namespace
+
+bool UserSegmentHistoryRewriter::SortCandidates(
+    const vector<ScoreType> &sorted_scores, Segment *segment) const {
   const uint32 top_score = sorted_scores[0].score;
   const size_t kMaxRerankSize = min(sorted_scores.size(),
                                     static_cast<size_t>(5));
@@ -447,6 +493,12 @@ bool SortCandidates(const vector<ScoreType> &sorted_scores, Segment *segment) {
         CharacterFormManager::GetCharacterFormManager()->
             ConvertConversionString(candidate->content_value,
                                     &(new_candidate->content_value));
+        // Update description so it matches candidate's current value.
+        // This fix addresses Bug #3493644.
+        // (Wrong character width annotation after learning alphabet)
+        new_candidate->description.clear();
+        VariantsRewriter::SetDescriptionForCandidate(*pos_matcher_,
+                                                     new_candidate);
         ++next_pos;
         seen.insert(normalized_value);
       }
@@ -460,17 +512,15 @@ bool SortCandidates(const vector<ScoreType> &sorted_scores, Segment *segment) {
   }
   return true;
 }
-}  // end of anonymous
 
-UserSegmentHistoryRewriter::UserSegmentHistoryRewriter()
-    : storage_(new LRUStorage) {
-  static const char kFileName[] = "user://segment.db";
-  const string filename = ConfigFileStream::GetFileName(kFileName);
-  if (!storage_->OpenOrCreate(filename.c_str(),
-                              kValueSize, kLRUSize, kSeedValue)) {
-    LOG(WARNING) << "cannot initialize UserSegmentHistoryRewriter";
-    storage_.reset(NULL);
-  }
+UserSegmentHistoryRewriter::UserSegmentHistoryRewriter(
+    const POSMatcher *pos_matcher,
+    const PosGroup *pos_group)
+    : storage_(new LRUStorage),
+      pos_matcher_(pos_matcher),
+      pos_group_(pos_group) {
+  Reload();
+  g_lru_storage = storage_.get();
 
   CHECK_EQ(sizeof(uint32), sizeof(FeatureValue));
   CHECK_EQ(sizeof(uint32), sizeof(KeyTriggerValue));
@@ -507,11 +557,11 @@ do { \
 bool UserSegmentHistoryRewriter::GetScore(const Segments &segments,
                                           size_t segment_index,
                                           int candidate_index,
-                                          const string &top_functional_value,
-                                          uint16 top_pos_group,
                                           uint32 *score,
                                           uint32 *last_access_time) const {
   const size_t segments_size = segments.conversion_segments_size();
+  const Segment::Candidate &top_candidate =
+      segments.segment(segment_index).candidate(0);
   const Segment::Candidate &candidate =
       segments.segment(segment_index).candidate(candidate_index);
   const string &all_value = candidate.value;
@@ -526,10 +576,6 @@ bool UserSegmentHistoryRewriter::GetScore(const Segments &segments,
       (candidate.attributes & Segment::Candidate::CONTEXT_SENSITIVE) ||
       (segments.segment(segment_index).candidate(0).attributes &
        Segment::Candidate::CONTEXT_SENSITIVE);
-  const bool is_replaceable =
-      top_functional_value == candidate.functional_value() &&
-      top_pos_group == GetPosGroup(candidate);
-
   DCHECK(score);
   DCHECK(last_access_time);
 
@@ -555,6 +601,8 @@ bool UserSegmentHistoryRewriter::GetScore(const Segments &segments,
   FETCH_FEATURE(GetFeatureLN, content_key, content_value, bigram_number_score);
   FETCH_FEATURE(GetFeatureRN, content_key, content_value, bigram_number_score);
 
+  const bool is_replaceable = Replaceable(top_candidate, candidate);
+
   if (!context_sensitive && is_replaceable) {
     FETCH_FEATURE(GetFeatureC,  all_key, all_value, unigram_score);
   }
@@ -581,14 +629,26 @@ bool UserSegmentHistoryRewriter::GetScore(const Segments &segments,
   return (*score > 0);
 }
 
+// Returns true if |lhs| candidate can be replaceable with |rhs|.
+bool UserSegmentHistoryRewriter::Replaceable(
+    const Segment::Candidate &lhs, const Segment::Candidate &rhs) const {
+  const bool same_functional_value =
+      (lhs.functional_value() == rhs.functional_value());
+  const bool same_pos_group =
+      (pos_group_->GetPosGroup(lhs.lid) == pos_group_->GetPosGroup(rhs.lid));
+  return (same_functional_value &&
+          (same_pos_group || IsT13NCandidate(lhs) || IsT13NCandidate(rhs)));
+}
+
+
 void UserSegmentHistoryRewriter::RememberNumberPreference(
     const Segment &segment) {
   const Segment::Candidate &candidate = segment.candidate(0);
 
   if ((candidate.style ==
-       Segment::Candidate::NUMBER_SEPARATED_ARABIC_HALFWIDTH) ||
+       NumberUtil::NumberString::NUMBER_SEPARATED_ARABIC_HALFWIDTH) ||
       (candidate.style ==
-       Segment::Candidate::NUMBER_SEPARATED_ARABIC_FULLWIDTH)) {
+       NumberUtil::NumberString::NUMBER_SEPARATED_ARABIC_FULLWIDTH)) {
     // in the case of:
     // 1. submit "123"
     // 2. submit "一二三"
@@ -600,7 +660,7 @@ void UserSegmentHistoryRewriter::RememberNumberPreference(
     // separated and default is learned at same time
     // This problem is solved by workaround on lookup.
     string default_feature_key;
-    GetFeatureN(Segment::Candidate::DEFAULT_STYLE, &default_feature_key);
+    GetFeatureN(NumberUtil::NumberString::DEFAULT_STYLE, &default_feature_key);
     FeatureValue v;
     DCHECK(v.IsValid());
     storage_->Insert(default_feature_key, reinterpret_cast<const char *>(&v));
@@ -646,11 +706,8 @@ void UserSegmentHistoryRewriter::RememberFirstCandidate(
   // if "is_replaceable" is true, it means that  the target candidate can
   // "SAFELY" be replaceable with the top candidate.
   const int top_index = GetDefaultCandidateIndex(seg);
-  const bool is_replaceable =
-      (top_index == 0) ||
-      (GetPosGroup(seg.candidate(top_index)) == GetPosGroup(candidate) &&
-       (seg.candidate(top_index).functional_value() ==
-        candidate.functional_value()));
+  const bool is_replaceable_with_top =
+      ((top_index == 0) || Replaceable(seg.candidate(top_index), candidate));
 
   // |feature_key| is used inside INSERT_FEATURE
   string feature_key;
@@ -663,12 +720,14 @@ void UserSegmentHistoryRewriter::RememberFirstCandidate(
   INSERT_FEATURE(GetFeatureRN, all_key, all_value, force_insert);
   INSERT_FEATURE(GetFeatureS,  all_key, all_value, force_insert);
 
-  if (!context_sensitive && is_replaceable) {
+  if (!context_sensitive && is_replaceable_with_top) {
     INSERT_FEATURE(GetFeatureC, all_key, all_value, force_insert);
   }
 
   // save content value
-  if (all_value != content_value && all_key != content_key && is_replaceable) {
+  if (all_value != content_value &&
+      all_key != content_key &&
+      is_replaceable_with_top) {
     INSERT_FEATURE(GetFeatureLR, content_key, content_value, force_insert);
     INSERT_FEATURE(GetFeatureLL, content_key, content_value, force_insert);
     INSERT_FEATURE(GetFeatureRR, content_key, content_value, force_insert);
@@ -723,13 +782,17 @@ bool UserSegmentHistoryRewriter::IsAvailable(const Segments &segments) const {
   return true;
 }
 
-void UserSegmentHistoryRewriter::Finish(Segments *segments) {
+void UserSegmentHistoryRewriter::Finish(const ConversionRequest &request,
+                                        Segments *segments) {
+  if (segments->request_type() != Segments::CONVERSION) {
+    return;
+  }
+
   if (!IsAvailable(*segments)) {
     return;
   }
 
-  if (GET_CONFIG(history_learning_level) !=
-      config::Config::DEFAULT_HISTORY) {
+  if (GET_CONFIG(history_learning_level) != Config::DEFAULT_HISTORY) {
     VLOG(2) << "history_learning_level is not DEFAULT_HISTORY";
     return;
   }
@@ -755,6 +818,28 @@ void UserSegmentHistoryRewriter::Finish(Segments *segments) {
                                       static_cast<int>(storage_->used_size()));
 }
 
+bool UserSegmentHistoryRewriter::Reload() {
+  const string filename = ConfigFileStream::GetFileName(kFileName);
+  if (!storage_->OpenOrCreate(filename.c_str(),
+                              kValueSize, kLRUSize, kSeedValue)) {
+    LOG(WARNING) << "cannot initialize UserSegmentHistoryRewriter";
+    storage_.reset(NULL);
+    g_lru_storage = NULL;
+    return false;
+  }
+
+  const char kFileSuffix[] = ".merge_pending";
+  const string merge_pending_file = filename + kFileSuffix;
+
+  // merge pending file does not always exist.
+  if (FileUtil::FileExists(merge_pending_file)) {
+    storage_->Merge(merge_pending_file.c_str());
+    FileUtil::Unlink(merge_pending_file);
+  }
+
+  return true;
+}
+
 bool UserSegmentHistoryRewriter::ShouldRewrite(
     const Segment &segment,
     size_t *max_candidates_size) const {
@@ -793,7 +878,8 @@ void UserSegmentHistoryRewriter::InsertTriggerKey(const Segment &segment) {
   DCHECK(storage_.get());
 
   KeyTriggerValue v;
-  DCHECK_EQ(sizeof(v), sizeof(uint32));
+  static_assert(sizeof(uint32) == sizeof(v),
+                "KeyTriggerValue must be 32-bit int size.");
 
   // TODO(taku): saving segment.candidate_size() might be too heavy and
   // increases the chance of hash collisions.
@@ -836,9 +922,9 @@ bool UserSegmentHistoryRewriter::RewriteNumber(Segment *segment) const {
       // has higher rank by sorting of scores.
       if (last_access_time > 0 &&
           (segment->candidate(j).style
-           != Segment::Candidate::NUMBER_SEPARATED_ARABIC_FULLWIDTH) &&
+           != NumberUtil::NumberString::NUMBER_SEPARATED_ARABIC_FULLWIDTH) &&
           (segment->candidate(j).style
-           != Segment::Candidate::NUMBER_SEPARATED_ARABIC_HALFWIDTH)) {
+           != NumberUtil::NumberString::NUMBER_SEPARATED_ARABIC_HALFWIDTH)) {
         last_access_time--;
       }
       scores.resize(scores.size() + 1);
@@ -856,19 +942,19 @@ bool UserSegmentHistoryRewriter::RewriteNumber(Segment *segment) const {
   return SortCandidates(scores, segment);
 }
 
-bool UserSegmentHistoryRewriter::Rewrite(Segments *segments) const {
+bool UserSegmentHistoryRewriter::Rewrite(const ConversionRequest &request,
+                                         Segments *segments) const {
   if (!IsAvailable(*segments)) {
     return false;
   }
 
-  if (GET_CONFIG(history_learning_level) == config::Config::NO_HISTORY) {
+  if (GET_CONFIG(history_learning_level) == Config::NO_HISTORY) {
     VLOG(2) << "history_learning_level is NO_HISTORY";
     return false;
   }
 
   // set BEST_CANDIDATE marker in advance
-  for (size_t i = segments->history_segments_size();
-       i < segments->segments_size(); ++i) {
+  for (size_t i = 0; i < segments->segments_size(); ++i) {
     Segment *segment = segments->mutable_segment(i);
     DCHECK(segment);
     DCHECK_GT(segment->candidates_size(), 0);
@@ -901,14 +987,8 @@ bool UserSegmentHistoryRewriter::Rewrite(Segments *segments) const {
       continue;
     }
 
-    if (segment->candidates_size() < max_candidates_size) {
-      LOG(WARNING) << "cannot expand candidates. ignored."
-                   << "rewrite may be failed ";
-    }
-
-    const string top_functional_value =
-        segment->candidate(0).functional_value();
-    const uint16 top_pos_group = GetPosGroup(segment->candidate(0));
+    DVLOG_IF(2, (segment->candidates_size() < max_candidates_size))
+        << "Cannot expand candidates. ignored. Rewrite may be failed";
 
     // for each all candidates expanded
     vector<ScoreType> scores;
@@ -923,9 +1003,8 @@ bool UserSegmentHistoryRewriter::Rewrite(Segments *segments) const {
 
       uint32 score = 0;
       uint32 last_access_time = 0;
-      if (GetScore(*segments, i, j, top_functional_value, top_pos_group,
-                   &score, &last_access_time)) {
-        scores.resize(scores.size() + 1);
+      if (GetScore(*segments, i, j, &score, &last_access_time)) {
+        scores.push_back(ScoreType());
         scores.back().score = score;
         scores.back().last_access_time = last_access_time;
         scores.back().candidate = segment->mutable_candidate(j);
@@ -948,4 +1027,61 @@ void UserSegmentHistoryRewriter::Clear() {
     storage_->Clear();
   }
 }
-}  // namespace
+
+// static
+LRUStorage *UserSegmentHistoryRewriter::GetStorage() {
+  return g_lru_storage;
+}
+
+bool UserSegmentHistoryRewriter::IsPunctuation(
+    const Segment &seg,
+    const Segment::Candidate &candidate) const {
+  return (pos_matcher_->IsJapanesePunctuations(candidate.lid) &&
+          candidate.lid == candidate.rid &&
+          IsPunctuationInternal(seg.key()) &&
+          IsPunctuationInternal(candidate.value));
+}
+
+// Feature "Left Number"
+bool UserSegmentHistoryRewriter::GetFeatureLN(const Segments &segments,
+                                              size_t i,
+                                              const string &base_key,
+                                              const string &base_value,
+                                              string *value) const {
+  DCHECK(value);
+  if (i < 1) {
+    return false;
+  }
+  const int j = GetDefaultCandidateIndex(segments.segment(i - 1));
+  const Segment::Candidate &candidate = segments.segment(i - 1).candidate(j);
+  if (pos_matcher_->IsNumber(candidate.rid) ||
+      pos_matcher_->IsKanjiNumber(candidate.rid) ||
+      Util::GetScriptType(candidate.value) == Util::NUMBER) {
+    JoinStringsWithTab3(StringPiece("LN", 2), base_key, base_value, value);
+    return true;
+  }
+  return false;
+}
+
+// Feature "Right Number"
+bool UserSegmentHistoryRewriter::GetFeatureRN(const Segments &segments,
+                                              size_t i,
+                                              const string &base_key,
+                                              const string &base_value,
+                                              string *value) const {
+  DCHECK(value);
+  if (i + 1 >= segments.segments_size()) {
+    return false;
+  }
+  const int j = GetDefaultCandidateIndex(segments.segment(i + 1));
+  const Segment::Candidate &candidate = segments.segment(i + 1).candidate(j);
+  if (pos_matcher_->IsNumber(candidate.lid) ||
+      pos_matcher_->IsKanjiNumber(candidate.lid) ||
+      Util::GetScriptType(candidate.value) == Util::NUMBER) {
+    JoinStringsWithTab3(StringPiece("RN", 2), base_key, base_value, value);
+    return true;
+  }
+  return false;
+}
+
+}  // namespace mozc
