@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -27,7 +27,7 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
 
 #include "server/cache_service_manager.h"
 
@@ -37,16 +37,17 @@
 #include <wincrypt.h>
 #include <string>
 
-#include "base/base.h"
+#include "base/const.h"
+#include "base/file_util.h"
 #include "base/scoped_handle.h"
+#include "base/scoped_ptr.h"
+#include "base/system_util.h"
 #include "base/util.h"
 #include "server/mozc_cache_service_resource.h"
 #include "server/win32_service_state.pb.h"
 
 namespace mozc {
 namespace {
-const char    kProgramName[] = "GoogleIMEJaCacheService.exe";
-const wchar_t kServiceName[] = L"GoogleIMEJaCacheService";
 
 const uint64 kMinimumRequiredMemorySizeForInstall = 384 * 1024 * 1024;
 
@@ -85,13 +86,13 @@ wstring GetRegistryStringRedirectorOrRedirectedString(int resource_id) {
   wchar_t buffer[MAX_PATH] = { L'\0' };
 
   HRESULT hr = ::StringCchPrintf(
-      buffer, ARRAYSIZE(buffer), kRegistryStringRedirectionPattern,
+      buffer, arraysize(buffer), kRegistryStringRedirectionPattern,
       CacheServiceManager::GetUnquotedServicePath().c_str(), resource_id);
   if (hr != S_OK) {
     return L"";
   }
   const wstring redirector(buffer);
-  if (Util::IsVistaOrLater()) {
+  if (SystemUtil::IsVistaOrLater()) {
     // If this program is running on Windows Vista or later,
     // just returns the redirector.
     return redirector;
@@ -231,7 +232,7 @@ bool IsServiceRunning(const ScopedSCHandle &service_handle) {
     return false;
   }
 
-  SERVICE_STATUS service_status = { 0 };
+  SERVICE_STATUS service_status = {};
   if (!::QueryServiceStatus(service_handle.get(), &service_status)) {
     LOG(ERROR) << "QueryServiceStatus failed: " << ::GetLastError();
     return false;
@@ -279,7 +280,7 @@ bool SetServiceDescription(const ScopedSCHandle &service_handle,
   description._Copy_s(buffer.get(), buffer_length, description.size());
   buffer[buffer_length - 1] = L'\0';
 
-  SERVICE_DESCRIPTION desc = { 0 };
+  SERVICE_DESCRIPTION desc = {};
   desc.lpDescription = buffer.get();
   if (!::ChangeServiceConfig2(service_handle.get(),
                               SERVICE_CONFIG_DESCRIPTION, &desc)) {
@@ -287,40 +288,65 @@ bool SetServiceDescription(const ScopedSCHandle &service_handle,
     return false;
   }
 
-  // Windows Vista or later allows the SCM to run a service in a restricted
-  // context as described in following documents.
+  return true;
+}
+
+// Set *nice-to-have* features for the cache service.
+// We do not care about any failure because it is not critical for the
+// functionality of the cache service itself.
+void SetAdvancedConfig(const ScopedSCHandle &service_handle) {
+  // On Windows XP, we have nothing to do.
+  if (!SystemUtil::IsVistaOrLater()) {
+    return;
+  }
+
+  // Windows Vista or later has some nice features as described in the
+  // following documents.
   // http://msdn.microsoft.com/en-us/magazine/cc164252.aspx
+
+  // Enable "Delayed Auto-start".
+  {
+    SERVICE_DELAYED_AUTO_START_INFO info = {};
+    info.fDelayedAutostart = TRUE;
+
+    if (!::ChangeServiceConfig2(service_handle.get(),
+                                SERVICE_CONFIG_DELAYED_AUTO_START_INFO,
+                                &info)) {
+      LOG(ERROR) << "ChangeServiceConfig2 failed: " << ::GetLastError();
+    }
+  }
+
   // http://blogs.technet.com/richard_macdonald/archive/2007/06/27/1375523.aspx
   // See also http://b/2470180
-  if (Util::IsVistaOrLater()) {
-    // Only SE_INC_BASE_PRIORITY_NAME and SE_CHANGE_NOTIFY are needed.
-    // According to MSDN Library, we need not explicitly specify the later.
-    // See http://msdn.microsoft.com/en-us/library/ms685976.aspx for details.
-    SERVICE_REQUIRED_PRIVILEGES_INFO privileges_info = { 0 };
+
+  // Only SE_INC_BASE_PRIORITY_NAME and SE_CHANGE_NOTIFY are needed.
+  // According to MSDN Library, we need not explicitly specify the later.
+  // See http://msdn.microsoft.com/en-us/library/ms685976.aspx for details.
+  {
+    SERVICE_REQUIRED_PRIVILEGES_INFO privileges_info = {};
     privileges_info.pmszRequiredPrivileges = SE_INC_BASE_PRIORITY_NAME
                                              TEXT("\0");
     if (!::ChangeServiceConfig2(service_handle.get(),
                                 SERVICE_CONFIG_REQUIRED_PRIVILEGES_INFO,
                                 &privileges_info)) {
       LOG(ERROR) << "ChangeServiceConfig2 failed: " << ::GetLastError();
-      return false;
     }
+  }
 
-    // Remove write privileges from the cache service.
-    // See http://msdn.microsoft.com/en-us/library/ms685987.aspx for details.
-    // This may restrict glog functions such as LOG(ERROR).
-    // TODO(yukawa): Output logging messages as debug strings, or output them
-    // to the Win32 event log.
-    SERVICE_SID_INFO sid_info = { 0 };
+  // Remove write privileges from the cache service.
+  // See http://msdn.microsoft.com/en-us/library/ms685987.aspx for details.
+  // This may restrict glog functions such as LOG(ERROR).
+  // TODO(yukawa): Output logging messages as debug strings, or output them
+  // to the Win32 event log.
+  {
+    SERVICE_SID_INFO sid_info = {};
     sid_info.dwServiceSidType = SERVICE_SID_TYPE_RESTRICTED;
     if (!::ChangeServiceConfig2(service_handle.get(),
                                 SERVICE_CONFIG_SERVICE_SID_INFO,
                                 &sid_info)) {
       LOG(ERROR) << "ChangeServiceConfig2 failed: " << ::GetLastError();
-      return false;
     }
   }
-  return true;
 }
 
 // This function updates the following settings of the cache service as
@@ -353,6 +379,10 @@ bool RestoreStateInternal(const cache_service::Win32ServiceState &state) {
     return false;
   }
 
+  // We ignore any failure of advanced configurations because they are
+  // *nice-to-have* features.
+  SetAdvancedConfig(service_handle);
+
   const bool now_running = IsServiceRunning(service_handle);
 
   // If the service was runnning and is not running now, start it unless the
@@ -461,12 +491,13 @@ bool CacheServiceManager::IsEnabled() {
 }
 
 const wchar_t *CacheServiceManager::GetServiceName() {
-  return kServiceName;
+  return kMozcCacheServiceName;
 }
 
 wstring CacheServiceManager::GetUnquotedServicePath() {
   const string lock_service_path =
-      Util::JoinPath(Util::GetServerDirectory(), kProgramName);
+      FileUtil::JoinPath(SystemUtil::GetServerDirectory(),
+                         kMozcCacheServiceExeName);
   wstring wlock_service_path;
   if (Util::UTF8ToWide(lock_service_path.c_str(), &wlock_service_path) <= 0) {
     return L"";
@@ -532,7 +563,7 @@ bool CacheServiceManager::RestartService() {
 
   const int kNumTrial = 10;
   for (int i = 0; i < kNumTrial; ++i) {
-    SERVICE_STATUS service_status = { 0 };
+    SERVICE_STATUS service_status = {};
     if (!::QueryServiceStatus(service_handle.get(), &service_status)) {
       LOG(ERROR) << "QueryServiceStatus failed: " << ::GetLastError();
       return false;
@@ -547,8 +578,8 @@ bool CacheServiceManager::RestartService() {
 }
 
 bool CacheServiceManager::HasEnoughMemory() {
-  return Util::GetTotalPhysicalMemory()
-      >= kMinimumRequiredMemorySizeForInstall;
+  return SystemUtil::GetTotalPhysicalMemory() >=
+      kMinimumRequiredMemorySizeForInstall;
 }
 
 bool CacheServiceManager::BackupStateAsString(wstring *result) {
@@ -635,5 +666,6 @@ bool CacheServiceManager::EnsureServiceStopped() {
 
   return IsServiceRunning(service_handle);
 }
-}  // mozc
-#endif  // OS_WINDOWS
+
+}  // namespace mozc
+#endif  // OS_WIN
