@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -31,18 +31,22 @@
 
 #include "session/internal/keymap.h"
 
+#include <set>
+#include <sstream>
 #include <string>
 #include <vector>
-#include <sstream>
 
-#include "base/file_stream.h"
 #include "base/config_file_stream.h"
+#include "base/file_stream.h"
+#include "base/logging.h"
+#include "base/port.h"
+#include "base/scoped_ptr.h"
 #include "base/util.h"
-#include "session/config.pb.h"
-#include "session/config_handler.h"
+#include "config/config.pb.h"
+#include "config/config_handler.h"
 #include "session/commands.pb.h"
 #include "session/internal/keymap-inl.h"
-#include "session/key_event_normalizer.h"
+#include "session/key_event_util.h"
 #include "session/key_parser.h"
 
 namespace mozc {
@@ -52,143 +56,29 @@ static const char kMSIMEKeyMapFile[] = "system://ms-ime.tsv";
 static const char kATOKKeyMapFile[] = "system://atok.tsv";
 static const char kKotoeriKeyMapFile[] = "system://kotoeri.tsv";
 static const char kCustomKeyMapFile[] = "user://keymap.tsv";
+static const char kMobileKeyMapFile[] = "system://mobile.tsv";
+
+// TODO(team): Investigate if "InputModeX" commands should be
+//     functional on ChromeOS and/or Android or not. Remove the
+//     condition if available.
+#if defined(OS_MACOSX) || defined(OS_CHROMEOS) || defined(OS_ANDROID)
+const bool kInputModeXCommandSupported = false;
+#else
+const bool kInputModeXCommandSupported = true;
+#endif
 
-uint32 GetModifiers(const commands::KeyEvent &key_event) {
-  uint32 modifiers = 0;
-  if (key_event.has_modifiers()) {
-    modifiers = key_event.modifiers();
-  } else {
-    for (int i = 0; i < key_event.modifier_keys_size(); ++i) {
-      modifiers |= key_event.modifier_keys(i);
-    }
-  }
-  return modifiers;
-}
 }  // anonymous namespace
 
-bool GetKey(const commands::KeyEvent &key_event, Key *key) {
-  // Key is an alias of uint64.
-  Key modifier_keys = GetModifiers(key_event);
-  Key special_key = key_event.has_special_key() ?
-      key_event.special_key() : commands::KeyEvent::NO_SPECIALKEY;
-  Key key_code = key_event.has_key_code() ? key_event.key_code() : 0;
-
-  // Make sure the translation from the obsolete spesification.
-  // key_code should no longer contain control characters.
-  if (0 < key_code && key_code <= 32) {
-    return false;
-  }
-
-  // Key = |Modifiers(16bit)|SpecialKey(16bit)|Unicode(32bit)|.
-  *key = (modifier_keys << 48) + (special_key << 32) + key_code;
-  return true;
-}
-
-// Return a fallback keyevent generated from key_event.  In the
-// current implementation, if the input key_event does not contains
-// any special keys or modifier keys, that printable key will be
-// replaced the ASCII special key.
-bool MaybeGetKeyStub(const commands::KeyEvent &key_event, Key *key) {
-  // If any modifier keys were pressed, this function does nothing.
-  if (GetModifiers(key_event) != 0) {
-    return false;
-  }
-
-  // No stub rule is supported for special keys yet.
-  if (key_event.has_special_key()) {
-    return false;
-  }
-
-  if (!key_event.has_key_code() || key_event.key_code() <= 32) {
-    return false;
-  }
-
-  commands::KeyEvent stub_key_event;
-  stub_key_event.set_special_key(commands::KeyEvent::ASCII);
-  if (!GetKey(stub_key_event, key)) {
-    return false;
-  }
-
-  return true;
-}
-
-
 KeyMapManager::KeyMapManager()
     : keymap_(config::Config::NONE) {
   InitCommandData();
-  Reload();
+  ReloadWithKeymap(GET_CONFIG(session_keymap));
 }
 
 KeyMapManager::~KeyMapManager() {}
 
-void KeyMapManager::CheckIMEOnOffKeymap() {
-  uint64 key_on = 0, key_off = 0, key_eisu = 0;
-  {
-    commands::KeyEvent key_event;
-    KeyParser::ParseKey("ON", &key_event);
-    KeyEventNormalizer::ToUint64(key_event, &key_on);
-  }
-  {
-    commands::KeyEvent key_event;
-    KeyParser::ParseKey("OFF", &key_event);
-    KeyEventNormalizer::ToUint64(key_event, &key_off);
-  }
-  {
-    commands::KeyEvent key_event;
-    KeyParser::ParseKey("EISU", &key_event);
-    KeyEventNormalizer::ToUint64(key_event, &key_eisu);
-  }
-
-  if (key_on == 0 || key_off == 0 || key_eisu == 0) {
-    // One of KeyEventNormalizer fails: do nothing to avoid unexpected errors.
-    return;
-  }
-
-  bool need_to_be_migrated = true;
-  for (set<uint64>::const_iterator itr = ime_on_off_keys_.begin();
-       itr != ime_on_off_keys_.end(); ++itr) {
-    if (*itr != key_on && *itr != key_off && *itr != key_eisu) {
-      // This seems to have own settings.
-      need_to_be_migrated = false;
-      break;
-    }
-  }
-
-  if (need_to_be_migrated) {
-    // Add rules
-    commands::KeyEvent key_event_hankaku;
-    commands::KeyEvent key_event_kanji;
-    KeyParser::ParseKey("Hankaku/Zenkaku", &key_event_hankaku);
-    KeyParser::ParseKey("Kanji", &key_event_kanji);
-    keymap_direct_.AddRule(key_event_hankaku, DirectInputState::IME_ON);
-    keymap_precomposition_.AddRule(key_event_hankaku, PrecompositionState::IME_OFF);
-    keymap_composition_.AddRule(key_event_hankaku, CompositionState::IME_OFF);
-    keymap_conversion_.AddRule(key_event_hankaku, ConversionState::IME_OFF);
-    keymap_direct_.AddRule(key_event_kanji, DirectInputState::IME_ON);
-    keymap_precomposition_.AddRule(key_event_kanji, PrecompositionState::IME_OFF);
-    keymap_composition_.AddRule(key_event_kanji, CompositionState::IME_OFF);
-    keymap_conversion_.AddRule(key_event_kanji, ConversionState::IME_OFF);
-
-    // Write settings
-    config::Config config;
-    config.CopyFrom(config::ConfigHandler::GetConfig());
-    ostringstream oss(config.custom_keymap_table());
-    oss << endl;
-    oss << "DirectInput\tHankaku/Zenkaku\tIMEOn" << endl;
-    oss << "DirectInput\tKanji\tIMEOn" << endl;
-    oss << "Conversion\tHankaku/Zenkaku\tIMEOff" << endl;
-    oss << "Conversion\tKanji\tIMEOff" << endl;
-    oss << "Precomposition\tHankaku/Zenkaku\tIMEOff" << endl;
-    oss << "Precomposition\tKanji\tIMEOff" << endl;
-    oss << "Composition\tHankaku/Zenkaku\tIMEOff" << endl;
-    oss << "Composition\tKanji\tIMEOff" << endl;
-    config.set_custom_keymap_table(oss.str());
-    config::ConfigHandler::SetConfig(config);
-  }
-}
-
-bool KeyMapManager::Reload() {
-  const config::Config::SessionKeymap new_keymap = GET_CONFIG(session_keymap);
+bool KeyMapManager::ReloadWithKeymap(
+    const config::Config::SessionKeymap new_keymap) {
   // If the current keymap is the same with the new keymap and not
   // CUSTOM, do nothing.
   if (new_keymap == keymap_ && new_keymap != config::Config::CUSTOM) {
@@ -203,11 +93,10 @@ bool KeyMapManager::Reload() {
   keymap_precomposition_.Clear();
   keymap_composition_.Clear();
   keymap_conversion_.Clear();
+  keymap_zero_query_suggestion_.Clear();
   keymap_suggestion_.Clear();
   keymap_prediction_.Clear();
 
-  ime_on_off_keys_.clear();
-
   if (new_keymap == config::Config::CUSTOM) {
     const string &custom_keymap_table = GET_CONFIG(custom_keymap_table);
     if (custom_keymap_table.empty()) {
@@ -226,9 +115,7 @@ bool KeyMapManager::Reload() {
     }
 #endif
     istringstream ifs(custom_keymap_table);
-    const bool result = LoadStream(&ifs);
-    CheckIMEOnOffKeymap();
-    return result;
+    return LoadStream(&ifs);
   }
 
   if (keymap_file != NULL && LoadFile(keymap_file)) {
@@ -242,9 +129,11 @@ bool KeyMapManager::Reload() {
 // static
 const char *KeyMapManager::GetKeyMapFileName(
     const config::Config::SessionKeymap keymap) {
-  switch(keymap) {
+  switch (keymap) {
     case config::Config::ATOK:
       return kATOKKeyMapFile;
+    case config::Config::MOBILE:
+      return kMobileKeyMapFile;
     case config::Config::MSIME:
       return kMSIMEKeyMapFile;
     case config::Config::KOTOERI:
@@ -258,6 +147,7 @@ const char *KeyMapManager::GetKeyMapFileName(
                  << " appeared at key map initialization.";
       const config::Config::SessionKeymap default_keymap = GetDefaultKeyMap();
       DCHECK(default_keymap == config::Config::ATOK ||
+             default_keymap == config::Config::MOBILE ||
              default_keymap == config::Config::MSIME ||
              default_keymap == config::Config::KOTOERI ||
              default_keymap == config::Config::CUSTOM);
@@ -276,7 +166,7 @@ config::Config::SessionKeymap KeyMapManager::GetDefaultKeyMap() {
 }
 
 bool KeyMapManager::LoadFile(const char *filename) {
-  scoped_ptr<istream> ifs(ConfigFileStream::Open(filename));
+  scoped_ptr<istream> ifs(ConfigFileStream::LegacyOpen(filename));
   if (ifs.get() == NULL) {
     LOG(WARNING) << "cannot load keymap table: " << filename;
     return false;
@@ -307,89 +197,14 @@ bool KeyMapManager::LoadStreamWithErrors(istream *ifs, vector<string> *errors) {
       continue;
     }
 
-#ifdef NO_LOGGING  // means RELEASE BUILD
-    // On the release build, we do not support the Abort and ReportBug
-    // commands.
-    if (rules[2] == "Abort" || rules[2] == "ReportBug") {
-      continue;
-    }
-#endif  // NO_LOGGING
-
-#ifndef _DEBUG
-    // Only debug build supports the Abort command.
-    if (rules[2] == "Abort") {
-      continue;
-    }
-#endif  // NO_LOGGING
-
-    commands::KeyEvent key_event;
-    KeyParser::ParseKey(rules[1], &key_event);
-
-    // Migration code:
-    // check key events for IME ON/OFF
-    {
-      if (rules[2] == "IMEOn" ||
-          rules[2] == "IMEOff") {
-        uint64 key;
-        if (KeyEventNormalizer::ToUint64(key_event, &key)) {
-          ime_on_off_keys_.insert(key);
-        }
-      }
-    }
-
-    if (rules[0] == "DirectInput" || rules[0] == "Direct") {
-      DirectInputState::Commands command;
-      if (ParseCommandDirect(rules[2], &command)) {
-        keymap_direct_.AddRule(key_event, command);
-      } else {
-        LOG(ERROR) << "Unknown command: " << line;
-        errors->push_back(line);
-      }
-    } else if (rules[0] == "Precomposition") {
-      PrecompositionState::Commands command;
-      if (ParseCommandPrecomposition(rules[2], &command)) {
-        keymap_precomposition_.AddRule(key_event, command);
-      } else {
-        LOG(ERROR) << "Unknown command: " << line;
-        errors->push_back(line);
-      }
-    } else if (rules[0] == "Composition") {
-      CompositionState::Commands command;
-      if (ParseCommandComposition(rules[2], &command)) {
-        keymap_composition_.AddRule(key_event, command);
-      } else {
-        LOG(ERROR) << "Unknown command: " << line;
-        errors->push_back(line);
-      }
-    } else if (rules[0] == "Conversion") {
-      ConversionState::Commands command;
-      if (ParseCommandConversion(rules[2], &command)) {
-        keymap_conversion_.AddRule(key_event, command);
-      } else {
-        LOG(ERROR) << "Unknown command: " << line;
-        errors->push_back(line);
-      }
-    } else if (rules[0] == "Suggestion") {
-      CompositionState::Commands command;
-      if (ParseCommandComposition(rules[2], &command)) {
-        keymap_suggestion_.AddRule(key_event, command);
-      } else {
-        LOG(ERROR) << "Unknown command: " << line;
-        errors->push_back(line);
-      }
-    } else if (rules[0] == "Prediction") {
-      ConversionState::Commands command;
-      if (ParseCommandConversion(rules[2], &command)) {
-        keymap_prediction_.AddRule(key_event, command);
-      } else {
-        LOG(ERROR) << "Unknown command: " << line;
-        errors->push_back(line);
-      }
+    if (!AddCommand(rules[0], rules[1], rules[2])) {
+      errors->push_back(line);
+      LOG(ERROR) << "Unknown command: " << line;
     }
   }
 
   commands::KeyEvent key_event;
-  KeyParser::ParseKey("ASCII", &key_event);
+  KeyParser::ParseKey("TextInput", &key_event);
   keymap_precomposition_.AddRule(key_event,
                                  PrecompositionState::INSERT_CHARACTER);
   keymap_composition_.AddRule(key_event, CompositionState::INSERT_CHARACTER);
@@ -401,16 +216,114 @@ bool KeyMapManager::LoadStreamWithErrors(istream *ifs, vector<string> *errors) {
   return true;
 }
 
+bool KeyMapManager::AddCommand(const string &state_name,
+                               const string &key_event_name,
+                               const string &command_name) {
+#ifdef NO_LOGGING  // means RELEASE BUILD
+  // On the release build, we do not support the Abort and ReportBug
+  // commands.  Note, true is returned as the arguments are
+  // interpreted properly.
+  if (command_name == "Abort" || command_name == "ReportBug") {
+    return true;
+  }
+#endif  // NO_LOGGING
+
+#ifndef DEBUG
+  // Only debug build supports the Abort command.  Note, true is
+  // returned as the arguments are interpreted properly.
+  if (command_name == "Abort") {
+    return true;
+  }
+#endif  // DEBUG
+
+  commands::KeyEvent key_event;
+  if (!KeyParser::ParseKey(key_event_name, &key_event)) {
+    return false;
+  }
+
+  if (state_name == "DirectInput" || state_name == "Direct") {
+    DirectInputState::Commands command;
+    if (!ParseCommandDirect(command_name, &command)) {
+      return false;
+    }
+
+    keymap_direct_.AddRule(key_event, command);
+    return true;
+  }
+
+  if (state_name == "Precomposition") {
+    PrecompositionState::Commands command;
+    if (!ParseCommandPrecomposition(command_name, &command)) {
+      return false;
+    }
+
+    keymap_precomposition_.AddRule(key_event, command);
+    return true;
+  }
+
+  if (state_name == "Composition") {
+    CompositionState::Commands command;
+    if (!ParseCommandComposition(command_name, &command)) {
+      return false;
+    }
+
+    keymap_composition_.AddRule(key_event, command);
+    return true;
+  }
+
+  if (state_name == "Conversion") {
+    ConversionState::Commands command;
+    if (!ParseCommandConversion(command_name, &command)) {
+      return false;
+    }
+
+    keymap_conversion_.AddRule(key_event, command);
+    return true;
+  }
+
+  if (state_name == "ZeroQuerySuggestion") {
+    PrecompositionState::Commands command;
+    if (!ParseCommandPrecomposition(command_name, &command)) {
+      return false;
+    }
+
+    keymap_zero_query_suggestion_.AddRule(key_event, command);
+    return true;
+  }
+
+  if (state_name == "Suggestion") {
+    CompositionState::Commands command;
+    if (!ParseCommandComposition(command_name, &command)) {
+      return false;
+    }
+
+    keymap_suggestion_.AddRule(key_event, command);
+    return true;
+  }
+
+  if (state_name == "Prediction") {
+    ConversionState::Commands command;
+    if (!ParseCommandConversion(command_name, &command)) {
+      return false;
+    }
+
+    keymap_prediction_.AddRule(key_event, command);
+    return true;
+  }
+
+  return false;
+}
+
 namespace {
 template<typename T> bool GetNameInternal(
     const map<T, string> &reverse_command_map, T command, string *name) {
   DCHECK(name);
-  typename map<T, string>::const_iterator itr =
+  typename map<T, string>::const_iterator iter =
       reverse_command_map.find(command);
-  if (itr == reverse_command_map.end()) {
+  if (iter == reverse_command_map.end()) {
     return false;
   } else {
-    *name = itr->second;
+    *name = iter->second;
     return true;
   }
 }
@@ -469,7 +382,7 @@ void KeyMapManager::InitCommandData() {
   // Support InputMode command only on Windows for now.
   // TODO(toshiyuki): delete #ifdef when we support them on Mac, and
   // activate SessionTest.InputModeConsumedForTestSendKey.
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   RegisterDirectCommand("InputModeHiragana",
                         DirectInputState::INPUT_MODE_HIRAGANA);
   RegisterDirectCommand("InputModeFullKatakana",
@@ -480,7 +393,20 @@ void KeyMapManager::InitCommandData() {
                         DirectInputState::INPUT_MODE_FULL_ALPHANUMERIC);
   RegisterDirectCommand("InputModeHalfAlphanumeric",
                         DirectInputState::INPUT_MODE_HALF_ALPHANUMERIC);
-#endif  // OS_WINDOWS
+#else
+  RegisterDirectCommand("InputModeHiragana",
+                        DirectInputState::NONE);
+  RegisterDirectCommand("InputModeFullKatakana",
+                        DirectInputState::NONE);
+  RegisterDirectCommand("InputModeHalfKatakana",
+                        DirectInputState::NONE);
+  RegisterDirectCommand("InputModeFullAlphanumeric",
+                        DirectInputState::NONE);
+  RegisterDirectCommand("InputModeHalfAlphanumeric",
+                        DirectInputState::NONE);
+#endif  // OS_WIN
+  RegisterDirectCommand("Reconvert",
+                        DirectInputState::RECONVERT);
 
   // Precomposition
   RegisterPrecompositionCommand("IMEOff", PrecompositionState::IME_OFF);
@@ -497,26 +423,62 @@ void KeyMapManager::InitCommandData() {
                                 PrecompositionState::INSERT_FULL_SPACE);
   RegisterPrecompositionCommand("ToggleAlphanumericMode",
                                 PrecompositionState::TOGGLE_ALPHANUMERIC_MODE);
-#ifdef OS_WINDOWS
-  RegisterPrecompositionCommand("InputModeHiragana",
-                                PrecompositionState::INPUT_MODE_HIRAGANA);
-  RegisterPrecompositionCommand("InputModeFullKatakana",
-                                PrecompositionState::INPUT_MODE_FULL_KATAKANA);
-  RegisterPrecompositionCommand("InputModeHalfKatakana",
-                                PrecompositionState::INPUT_MODE_HALF_KATAKANA);
-  RegisterPrecompositionCommand(
-      "InputModeFullAlphanumeric",
-      PrecompositionState::INPUT_MODE_FULL_ALPHANUMERIC);
-  RegisterPrecompositionCommand(
-      "InputModeHalfAlphanumeric",
-      PrecompositionState::INPUT_MODE_HALF_ALPHANUMERIC);
-#endif  // OS_WINDOWS
-  RegisterPrecompositionCommand("Revert", PrecompositionState::REVERT);
-//  RegisterPrecompositionCommand("Undo", PrecompositionState::UNDO);
+  if (kInputModeXCommandSupported) {
+    RegisterPrecompositionCommand("InputModeHiragana",
+                                  PrecompositionState::INPUT_MODE_HIRAGANA);
+    RegisterPrecompositionCommand(
+        "InputModeFullKatakana",
+        PrecompositionState::INPUT_MODE_FULL_KATAKANA);
+    RegisterPrecompositionCommand(
+        "InputModeHalfKatakana",
+        PrecompositionState::INPUT_MODE_HALF_KATAKANA);
+    RegisterPrecompositionCommand(
+        "InputModeFullAlphanumeric",
+        PrecompositionState::INPUT_MODE_FULL_ALPHANUMERIC);
+    RegisterPrecompositionCommand(
+        "InputModeHalfAlphanumeric",
+        PrecompositionState::INPUT_MODE_HALF_ALPHANUMERIC);
+    RegisterPrecompositionCommand(
+        "InputModeSwitchKanaType",
+        PrecompositionState::INPUT_MODE_SWITCH_KANA_TYPE);
+  } else {
+    RegisterPrecompositionCommand("InputModeHiragana",
+                                  PrecompositionState::NONE);
+    RegisterPrecompositionCommand("InputModeFullKatakana",
+                                  PrecompositionState::NONE);
+    RegisterPrecompositionCommand("InputModeHalfKatakana",
+                                  PrecompositionState::NONE);
+    RegisterPrecompositionCommand("InputModeFullAlphanumeric",
+                                  PrecompositionState::NONE);
+    RegisterPrecompositionCommand("InputModeHalfAlphanumeric",
+                                  PrecompositionState::NONE);
+    RegisterPrecompositionCommand("InputModeSwitchKanaType",
+                                  PrecompositionState::NONE);
+  }
 
-#ifdef _DEBUG  // only for debugging
+  RegisterPrecompositionCommand("LaunchConfigDialog",
+                                PrecompositionState::LAUNCH_CONFIG_DIALOG);
+  RegisterPrecompositionCommand("LaunchDictionaryTool",
+                                PrecompositionState::LAUNCH_DICTIONARY_TOOL);
+  RegisterPrecompositionCommand(
+      "LaunchWordRegisterDialog",
+      PrecompositionState::LAUNCH_WORD_REGISTER_DIALOG);
+
+  RegisterPrecompositionCommand("Revert", PrecompositionState::REVERT);
+  RegisterPrecompositionCommand("Undo", PrecompositionState::UNDO);
+  RegisterPrecompositionCommand("Reconvert", PrecompositionState::RECONVERT);
+
+  RegisterPrecompositionCommand("Cancel", PrecompositionState::CANCEL);
+  RegisterPrecompositionCommand("CancelAndIMEOff",
+                                PrecompositionState::CANCEL_AND_IME_OFF);
+  RegisterPrecompositionCommand("CommitFirstSuggestion",
+                                PrecompositionState::COMMIT_FIRST_SUGGESTION);
+  RegisterPrecompositionCommand("PredictAndConvert",
+                                PrecompositionState::PREDICT_AND_CONVERT);
+
+#ifdef DEBUG  // only for debugging
   RegisterPrecompositionCommand("Abort", PrecompositionState::ABORT);
-#endif  // _DEBUG
+#endif  // DEBUG
 
   // Composition
   RegisterCompositionCommand("IMEOff", CompositionState::IME_OFF);
@@ -525,11 +487,18 @@ void KeyMapManager::InitCommandData() {
                              CompositionState::INSERT_CHARACTER);
   RegisterCompositionCommand("Delete", CompositionState::DEL);
   RegisterCompositionCommand("Backspace", CompositionState::BACKSPACE);
+  RegisterCompositionCommand("InsertSpace",
+                             CompositionState::INSERT_SPACE);
+  RegisterCompositionCommand("InsertAlternateSpace",
+                             CompositionState::INSERT_ALTERNATE_SPACE);
   RegisterCompositionCommand("InsertHalfSpace",
                              CompositionState::INSERT_HALF_SPACE);
   RegisterCompositionCommand("InsertFullSpace",
                              CompositionState::INSERT_FULL_SPACE);
   RegisterCompositionCommand("Cancel", CompositionState::CANCEL);
+  RegisterCompositionCommand("CancelAndIMEOff",
+                             CompositionState::CANCEL_AND_IME_OFF);
+  RegisterCompositionCommand("Undo", CompositionState::UNDO);
   RegisterCompositionCommand("MoveCursorLeft",
                              CompositionState::MOVE_CURSOR_LEFT);
   RegisterCompositionCommand("MoveCursorRight",
@@ -574,32 +543,50 @@ void KeyMapManager::InitCommandData() {
                              CompositionState::TRANSLATE_HALF_ASCII);
   RegisterCompositionCommand("ToggleAlphanumericMode",
                              CompositionState::TOGGLE_ALPHANUMERIC_MODE);
-#ifdef OS_WINDOWS
-  RegisterCompositionCommand("InputModeHiragana",
-                             CompositionState::INPUT_MODE_HIRAGANA);
-  RegisterCompositionCommand("InputModeFullKatakana",
-                             CompositionState::INPUT_MODE_FULL_KATAKANA);
-  RegisterCompositionCommand("InputModeHalfKatakana",
-                             CompositionState::INPUT_MODE_HALF_KATAKANA);
-  RegisterCompositionCommand("InputModeFullAlphanumeric",
-                             CompositionState::INPUT_MODE_FULL_ALPHANUMERIC);
-  RegisterCompositionCommand("InputModeHalfAlphanumeric",
-                             CompositionState::INPUT_MODE_HALF_ALPHANUMERIC);
-#endif  // OS_WINDOWS
-#ifdef _DEBUG  // only for debugging
+  if (kInputModeXCommandSupported) {
+    RegisterCompositionCommand("InputModeHiragana",
+                               CompositionState::INPUT_MODE_HIRAGANA);
+    RegisterCompositionCommand("InputModeFullKatakana",
+                               CompositionState::INPUT_MODE_FULL_KATAKANA);
+    RegisterCompositionCommand("InputModeHalfKatakana",
+                               CompositionState::INPUT_MODE_HALF_KATAKANA);
+    RegisterCompositionCommand("InputModeFullAlphanumeric",
+                               CompositionState::INPUT_MODE_FULL_ALPHANUMERIC);
+    RegisterCompositionCommand("InputModeHalfAlphanumeric",
+                               CompositionState::INPUT_MODE_HALF_ALPHANUMERIC);
+  } else {
+    RegisterCompositionCommand("InputModeHiragana",
+                             CompositionState::NONE);
+    RegisterCompositionCommand("InputModeFullKatakana",
+                               CompositionState::NONE);
+    RegisterCompositionCommand("InputModeHalfKatakana",
+                               CompositionState::NONE);
+    RegisterCompositionCommand("InputModeFullAlphanumeric",
+                               CompositionState::NONE);
+    RegisterCompositionCommand("InputModeHalfAlphanumeric",
+                               CompositionState::NONE);
+  }
+#ifdef DEBUG  // only for debugging
   RegisterCompositionCommand("Abort", CompositionState::ABORT);
-#endif  // _DEBUG
+#endif  // DEBUG
 
   // Conversion
   RegisterConversionCommand("IMEOff", ConversionState::IME_OFF);
   RegisterConversionCommand("IMEOn", ConversionState::IME_ON);
   RegisterConversionCommand("InsertCharacter",
                             ConversionState::INSERT_CHARACTER);
+  RegisterConversionCommand("InsertSpace",
+                            ConversionState::INSERT_SPACE);
+  RegisterConversionCommand("InsertAlternateSpace",
+                            ConversionState::INSERT_ALTERNATE_SPACE);
   RegisterConversionCommand("InsertHalfSpace",
                             ConversionState::INSERT_HALF_SPACE);
   RegisterConversionCommand("InsertFullSpace",
                             ConversionState::INSERT_FULL_SPACE);
   RegisterConversionCommand("Cancel", ConversionState::CANCEL);
+  RegisterConversionCommand("CancelAndIMEOff",
+                            ConversionState::CANCEL_AND_IME_OFF);
+  RegisterConversionCommand("Undo", ConversionState::UNDO);
   RegisterConversionCommand("SegmentFocusLeft",
                             ConversionState::SEGMENT_FOCUS_LEFT);
   RegisterConversionCommand("SegmentFocusRight",
@@ -651,28 +638,39 @@ void KeyMapManager::InitCommandData() {
                             ConversionState::TRANSLATE_FULL_ASCII);
   RegisterConversionCommand("DisplayAsHalfAlphanumeric",
                             ConversionState::TRANSLATE_HALF_ASCII);
-#ifdef OS_WINDOWS
-  RegisterConversionCommand("InputModeHiragana",
-                            ConversionState::INPUT_MODE_HIRAGANA);
-  RegisterConversionCommand("InputModeFullKatakana",
-                            ConversionState::INPUT_MODE_FULL_KATAKANA);
-  RegisterConversionCommand("InputModeHalfKatakana",
-                            ConversionState::INPUT_MODE_HALF_KATAKANA);
-  RegisterConversionCommand("InputModeFullAlphanumeric",
-                            ConversionState::INPUT_MODE_FULL_ALPHANUMERIC);
-  RegisterConversionCommand("InputModeHalfAlphanumeric",
-                            ConversionState::INPUT_MODE_HALF_ALPHANUMERIC);
-#endif  // OS_WINDOWS
+  RegisterConversionCommand("DeleteSelectedCandidate",
+                            ConversionState::DELETE_SELECTED_CANDIDATE);
+  if (kInputModeXCommandSupported) {
+    RegisterConversionCommand("InputModeHiragana",
+                              ConversionState::INPUT_MODE_HIRAGANA);
+    RegisterConversionCommand("InputModeFullKatakana",
+                              ConversionState::INPUT_MODE_FULL_KATAKANA);
+    RegisterConversionCommand("InputModeHalfKatakana",
+                              ConversionState::INPUT_MODE_HALF_KATAKANA);
+    RegisterConversionCommand("InputModeFullAlphanumeric",
+                              ConversionState::INPUT_MODE_FULL_ALPHANUMERIC);
+    RegisterConversionCommand("InputModeHalfAlphanumeric",
+                              ConversionState::INPUT_MODE_HALF_ALPHANUMERIC);
+  } else {
+    RegisterConversionCommand("InputModeHiragana",
+                              ConversionState::NONE);
+    RegisterConversionCommand("InputModeFullKatakana",
+                              ConversionState::NONE);
+    RegisterConversionCommand("InputModeHalfKatakana",
+                              ConversionState::NONE);
+    RegisterConversionCommand("InputModeFullAlphanumeric",
+                              ConversionState::NONE);
+    RegisterConversionCommand("InputModeHalfAlphanumeric",
+                              ConversionState::NONE);
+  }
 #ifndef NO_LOGGING  // means NOT RELEASE build
   RegisterConversionCommand("ReportBug", ConversionState::REPORT_BUG);
 #endif  // NO_LOGGING
-#ifdef _DEBUG  // only for dubugging
+#ifdef DEBUG  // only for dubugging
   RegisterConversionCommand("Abort", ConversionState::ABORT);
-#endif  // _DEBUG
+#endif  // DEBUG
 }
 
-#undef ADD_TO_COMMAND_MAP
-
 bool KeyMapManager::GetCommandDirect(
     const commands::KeyEvent &key_event,
     DirectInputState::Commands *command) const {
@@ -691,6 +689,17 @@ bool KeyMapManager::GetCommandComposition(
   return keymap_composition_.GetCommand(key_event, command);
 }
 
+bool KeyMapManager::GetCommandZeroQuerySuggestion(
+    const commands::KeyEvent &key_event,
+    PrecompositionState::Commands *command) const {
+  // try zero query suggestion rule first
+  if (keymap_zero_query_suggestion_.GetCommand(key_event, command)) {
+    return true;
+  }
+  // use precomposition rule
+  return keymap_precomposition_.GetCommand(key_event, command);
+}
+
 bool KeyMapManager::GetCommandSuggestion(
     const commands::KeyEvent &key_event,
     CompositionState::Commands *command) const {
@@ -698,7 +707,7 @@ bool KeyMapManager::GetCommandSuggestion(
   if (keymap_suggestion_.GetCommand(key_event, command)) {
     return true;
   }
-  // use preedit rule
+  // use composition rule
   return keymap_composition_.GetCommand(key_event, command);
 }
 
@@ -764,43 +773,48 @@ bool KeyMapManager::ParseCommandConversion(
 void KeyMapManager::GetAvailableCommandNameDirect(
     set<string> *command_names) const {
   DCHECK(command_names);
-  for (map<string, DirectInputState::Commands>::const_iterator itr
+  for (map<string, DirectInputState::Commands>::const_iterator iter
            = command_direct_map_.begin();
-       itr != command_direct_map_.end(); ++itr) {
-    command_names->insert(itr->first);
+       iter != command_direct_map_.end(); ++iter) {
+    command_names->insert(iter->first);
   }
 }
 
 void KeyMapManager::GetAvailableCommandNamePrecomposition(
     set<string> *command_names) const {
   DCHECK(command_names);
-  for (map<string, PrecompositionState::Commands>::const_iterator itr
+  for (map<string, PrecompositionState::Commands>::const_iterator iter
            = command_precomposition_map_.begin();
-       itr != command_precomposition_map_.end(); ++itr) {
-    command_names->insert(itr->first);
+       iter != command_precomposition_map_.end(); ++iter) {
+    command_names->insert(iter->first);
   }
 }
 
 void KeyMapManager::GetAvailableCommandNameComposition(
     set<string> *command_names) const {
   DCHECK(command_names);
-  for (map<string, CompositionState::Commands>::const_iterator itr
+  for (map<string, CompositionState::Commands>::const_iterator iter
            = command_composition_map_.begin();
-       itr != command_composition_map_.end(); ++itr) {
-    command_names->insert(itr->first);
+       iter != command_composition_map_.end(); ++iter) {
+    command_names->insert(iter->first);
   }
 }
 
 void KeyMapManager::GetAvailableCommandNameConversion(
     set<string> *command_names) const {
   DCHECK(command_names);
-  for (map<string, ConversionState::Commands>::const_iterator itr
+  for (map<string, ConversionState::Commands>::const_iterator iter
            = command_conversion_map_.begin();
-       itr != command_conversion_map_.end(); ++itr) {
-    command_names->insert(itr->first);
+       iter != command_conversion_map_.end(); ++iter) {
+    command_names->insert(iter->first);
   }
 }
 
+void KeyMapManager::GetAvailableCommandNameZeroQuerySuggestion(
+    set<string> *command_names) const {
+  GetAvailableCommandNamePrecomposition(command_names);
+}
+
 void KeyMapManager::GetAvailableCommandNameSuggestion(
     set<string> *command_names) const {
   GetAvailableCommandNameComposition(command_names);
