@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,7 +29,11 @@
 
 #include "session/random_keyevents_generator.h"
 
+#include <string>
+#include <vector>
+
 #include "base/base.h"
+#include "base/logging.h"
 #include "base/mutex.h"
 #include "base/util.h"
 
@@ -39,6 +43,10 @@ namespace {
 
 #include "session/session_stress_test_data.h"
 
+// Constants for ProbableKeyEvent.
+const double kMostPossibleKeyProbability = 0.98;
+const size_t kProbableKeyEventSize = 8;
+
 const commands::KeyEvent::SpecialKey kSpecialKeys[] = {
   commands::KeyEvent::SPACE,
   commands::KeyEvent::BACKSPACE,
@@ -86,40 +94,129 @@ const commands::KeyEvent::SpecialKey kSpecialKeys[] = {
   commands::KeyEvent::SUBTRACT,
   commands::KeyEvent::DECIMAL,
   commands::KeyEvent::DIVIDE,
-  commands::KeyEvent::EQUALS
+  commands::KeyEvent::EQUALS,
+  commands::KeyEvent::COMMA,
 };
 
-int GetRandom(int size) {
-  return static_cast<int> (1.0 * size * rand() / (RAND_MAX + 1.0));
-}
-
 uint32 GetRandomAsciiKey() {
-  return static_cast<uint32>(' ') + GetRandom(static_cast<uint32>('~' - ' '));
+  return static_cast<uint32>(' ') +
+      Util::Random(static_cast<uint32>('~' - ' '));
 }
 
-void InitSeed() {
+void InitSeedWithRandomValue() {
   uint32 seed = 0;
-  mozc::Util::GetSecureRandomSequence(reinterpret_cast<char *>(&seed),
-                                      sizeof(seed));
-  srand(seed);
+  mozc::Util::GetRandomSequence(reinterpret_cast<char *>(&seed), sizeof(seed));
+  Util::SetRandomSeed(seed);
+}
+
+void DoNothing() {
+  // Do nothing.
+  // Used only for marking the seed initialized.
 }
 
 once_t seed_init_once = MOZC_ONCE_INIT;
 }  // namespace
 
+void RandomKeyEventsGenerator::PrepareForMemoryLeakTest() {
+  // Read all kTestSentences and load these to memory.
+  const int size = arraysize(kTestSentences);
+  for (int i = 0; i < size; ++i) {
+    const char *sentence = kTestSentences[i];
+    CHECK_GT(strlen(sentence), 0);
+  }
+}
+
+// Generates KeyEvent instances based on |romaji| and stores into |keys|.
+void TypeRawKeys(StringPiece romaji, bool create_probable_key_events,
+                 vector<commands::KeyEvent> *keys) {
+  for (ConstChar32Iterator iter(romaji); !iter.Done(); iter.Next()) {
+    const uint32 ucs4 = iter.Get();
+    if (ucs4 < 0x20 || ucs4 > 0x7F) {
+      continue;
+    }
+    commands::KeyEvent key;
+    key.set_key_code(ucs4);
+    if (create_probable_key_events) {
+      commands::KeyEvent::ProbableKeyEvent *probable_key_event =
+          key.add_probable_key_event();
+        probable_key_event->set_key_code(ucs4);
+        probable_key_event->set_probability(kMostPossibleKeyProbability);
+      for (size_t i = 0; i < kProbableKeyEventSize; ++i) {
+        commands::KeyEvent::ProbableKeyEvent *probable_key_event =
+            key.add_probable_key_event();
+        probable_key_event->set_key_code(0x20 + Util::Random(0x7F - 0x20));
+        probable_key_event->set_probability(
+            (1.0 - kMostPossibleKeyProbability) / kProbableKeyEventSize);
+      }
+    }
+    keys->push_back(key);
+  }
+}
+
+// Converts from Hiragana to Romaji.
+string ToRomaji(StringPiece hiragana) {
+  string tmp, result;
+  Util::HiraganaToRomanji(hiragana, &tmp);
+  Util::FullWidthToHalfWidth(tmp, &result);
+  return result;
+}
+
+void RandomKeyEventsGenerator::InitSeed(uint32 seed) {
+  Util::SetRandomSeed(seed);
+  CallOnce(&seed_init_once, &DoNothing);
+}
+
+// Generates KeyEvent instances based on |sentence| and stores into |keys|.
+// And Enter key event is appended at the tail.
+// The instances have ProbableKeyEvent if |create_probable_key_events| is set.
+void GenerateMobileSequenceInternal(
+    StringPiece sentence, bool create_probable_key_events,
+    vector<commands::KeyEvent> *keys) {
+  const string input = ToRomaji(sentence);
+  VLOG(1) << input;
+
+  // Type the sentence
+  TypeRawKeys(input, create_probable_key_events, keys);
+
+  commands::KeyEvent key;
+  key.set_special_key(commands::KeyEvent::ENTER);
+  keys->push_back(key);
+}
+
+void RandomKeyEventsGenerator::GenerateMobileSequence(
+    bool create_probable_key_events, vector<commands::KeyEvent> *keys) {
+  CHECK(keys);
+  keys->clear();
+
+  // If seed was not initialized, set seed randomly.
+  CallOnce(&seed_init_once, &InitSeedWithRandomValue);
+
+  const StringPiece sentence(
+      kTestSentences[Util::Random(arraysize(kTestSentences))]);
+  CHECK(!sentence.empty());
+  for (size_t i = 0; i < sentence.size(); ) {
+    // To simulate mobile key events, split given sentence into smaller parts.
+    // Average 5, Min 1, Max 15
+    const size_t len = Util::Random(5) + Util::Random(5) + Util::Random(5);
+    GenerateMobileSequenceInternal(sentence.substr(i, len),
+                                   create_probable_key_events, keys);
+    i += len;
+  }
+}
+
 void RandomKeyEventsGenerator::GenerateSequence(
     vector<commands::KeyEvent> *keys) {
   CHECK(keys);
   keys->clear();
 
-  CallOnce(&seed_init_once, &InitSeed);
+  // If seed was not initialized, set seed randomly.
+  CallOnce(&seed_init_once, &InitSeedWithRandomValue);
 
-  const string sentence = kTestSentences[GetRandom(arraysize(kTestSentences))];
+  const string sentence =
+      kTestSentences[Util::Random(arraysize(kTestSentences))];
   CHECK(!sentence.empty());
 
-  string tmp, input;
-  Util::HiraganaToRomanji(sentence, &tmp);
-  Util::FullWidthToHalfWidth(tmp, &input);
+  const string input = ToRomaji(sentence);
 
   VLOG(1) << input;
 
@@ -133,22 +230,7 @@ void RandomKeyEventsGenerator::GenerateSequence(
   vector<commands::KeyEvent> basic_keys;
 
   // generate basic input
-  {
-    // first add a normal keys
-    const char *begin = input.data();
-    const char *end = input.data() + input.size();
-    while (begin < end) {
-      size_t mblen = 0;
-      const uint16 ucs2 = Util::UTF8ToUCS2(begin, end, &mblen);
-      CHECK_GT(mblen, 0);
-      begin += mblen;
-      if (ucs2 >= 0x20 && ucs2 <= 0x7F) {
-        commands::KeyEvent key;
-        key.set_key_code(ucs2);
-        basic_keys.push_back(key);
-      }
-    }
-  }
+  TypeRawKeys(input, false, &basic_keys);
 
   // basic keys + conversion
   {
@@ -157,11 +239,11 @@ void RandomKeyEventsGenerator::GenerateSequence(
     }
 
     for (int i = 0; i < 5; ++i) {
-      const size_t num = GetRandom(30) + 8;
+      const size_t num = Util::Random(30) + 8;
       for (size_t j = 0; j < num; ++j) {
         commands::KeyEvent key;
         key.set_special_key(commands::KeyEvent::SPACE);
-        if (GetRandom(4) == 0) {
+        if (Util::Random(4) == 0) {
           key.add_modifier_keys(commands::KeyEvent::SHIFT);
           keys->push_back(key);
         }
@@ -182,25 +264,25 @@ void RandomKeyEventsGenerator::GenerateSequence(
       keys->push_back(basic_keys[i]);
     }
 
-    const size_t num = GetRandom(30) + 10;
+    const size_t num = Util::Random(30) + 10;
     for (size_t i = 0; i < num; ++i) {
       commands::KeyEvent key;
-      switch (GetRandom(4)) {
+      switch (Util::Random(4)) {
         case 0:
           key.set_special_key(commands::KeyEvent::LEFT);
-          if (GetRandom(2) == 0) {
+          if (Util::Random(2) == 0) {
             key.add_modifier_keys(commands::KeyEvent::SHIFT);
           }
           break;
         case 1:
           key.set_special_key(commands::KeyEvent::RIGHT);
-          if (GetRandom(2) == 0) {
+          if (Util::Random(2) == 0) {
             key.add_modifier_keys(commands::KeyEvent::SHIFT);
           }
           break;
         default:
           {
-            const size_t space_num = GetRandom(20) + 3;
+            const size_t space_num = Util::Random(20) + 3;
             for (size_t i = 0; i < space_num; ++i) {
               key.set_special_key(commands::KeyEvent::SPACE);
               keys->push_back(key);
@@ -209,11 +291,11 @@ void RandomKeyEventsGenerator::GenerateSequence(
           break;
       }
 
-      if (GetRandom(4) == 0) {
+      if (Util::Random(4) == 0) {
         key.add_modifier_keys(commands::KeyEvent::CTRL);
       }
 
-      if (GetRandom(10) == 0) {
+      if (Util::Random(10) == 0) {
         key.add_modifier_keys(commands::KeyEvent::ALT);
       }
 
@@ -231,10 +313,10 @@ void RandomKeyEventsGenerator::GenerateSequence(
       keys->push_back(basic_keys[i]);
     }
 
-    const size_t num = GetRandom(20) + 10;
+    const size_t num = Util::Random(20) + 10;
     for (size_t i = 0; i < num; ++i) {
       commands::KeyEvent key;
-      switch (GetRandom(5)) {
+      switch (Util::Random(5)) {
         case 0:
           key.set_special_key(commands::KeyEvent::LEFT);
           break;
@@ -250,7 +332,7 @@ void RandomKeyEventsGenerator::GenerateSequence(
         default:
           {
             // add any ascii
-            const size_t insert_num = GetRandom(5) + 1;
+            const size_t insert_num = Util::Random(5) + 1;
             for (size_t i = 0; i < insert_num; ++i) {
               key.set_key_code(GetRandomAsciiKey());
             }
@@ -268,9 +350,9 @@ void RandomKeyEventsGenerator::GenerateSequence(
   {
     for (size_t i = 0; i < basic_keys.size(); ++i) {
       commands::KeyEvent key;
-      switch (GetRandom(8)) {
+      switch (Util::Random(8)) {
         case 0:
-          key.set_key_code(kSpecialKeys[GetRandom(arraysize(kSpecialKeys))]);
+          key.set_key_code(kSpecialKeys[Util::Random(arraysize(kSpecialKeys))]);
           break;
         case 1:
           key.set_key_code(GetRandomAsciiKey());
@@ -280,19 +362,19 @@ void RandomKeyEventsGenerator::GenerateSequence(
           break;
       }
 
-      if (GetRandom(10) == 0) {  // 10%
+      if (Util::Random(10) == 0) {  // 10%
         key.add_modifier_keys(commands::KeyEvent::CTRL);
       }
 
-      if (GetRandom(10) == 0) {  // 10%
+      if (Util::Random(10) == 0) {  // 10%
         key.add_modifier_keys(commands::KeyEvent::SHIFT);
       }
 
-      if (GetRandom(50) == 0) {  // 2%
+      if (Util::Random(50) == 0) {  // 2%
         key.add_modifier_keys(commands::KeyEvent::KEY_DOWN);
       }
 
-      if (GetRandom(50) == 0) {  // 2%
+      if (Util::Random(50) == 0) {  // 2%
         key.add_modifier_keys(commands::KeyEvent::KEY_UP);
       }
 
