@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,15 +29,18 @@
 
 #include "testing/base/public/googletest.h"
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
 #include <windows.h>
 #else
 #include <unistd.h>
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 
 #include <climits>
 #include <string>
+
+#include "base/file_util.h"
 #include "base/init.h"
+#include "base/logging.h"
 #include "base/util.h"
 
 DEFINE_string(test_srcdir, "",
@@ -51,14 +54,16 @@ DECLARE_string(program_invocation_name);
 namespace mozc {
 namespace {
 
-#ifdef OS_WINDOWS
+#include "testing/mozc_data_dir.h"
+
+#ifdef OS_WIN
 string GetProgramPath() {
   wchar_t w_path[MAX_PATH];
-  const DWORD char_size = GetModuleFileNameW(NULL, w_path, ARRAYSIZE(w_path));
+  const DWORD char_size = GetModuleFileNameW(NULL, w_path, arraysize(w_path));
   if (char_size == 0) {
     LOG(ERROR) << "GetModuleFileNameW failed.  error = " << ::GetLastError();
     return "";
-  } else if (char_size == ARRAYSIZE(w_path)) {
+  } else if (char_size >= arraysize(w_path)) {
     LOG(ERROR) << "The result of GetModuleFileNameW was truncated.";
     return "";
   }
@@ -68,22 +73,23 @@ string GetProgramPath() {
 }
 
 string GetTestSrcdir() {
-  // Return program path in default.
-  // TODO(toshiyuki) : Make it to be configured in build stage
-  return GetProgramPath();
+  const string srcdir(kMozcDataDir);
+  CHECK(FileUtil::DirectoryExists(srcdir)) << srcdir << " is not a directory.";
+  return srcdir;
 }
 
 string GetTestTmpdir() {
   const string tmpdir = GetProgramPath() + ".tmp";
 
-  if (!Util::DirectoryExists(tmpdir)) {
-    CHECK(Util::CreateDirectory(tmpdir));
+  if (!FileUtil::DirectoryExists(tmpdir)) {
+    CHECK(FileUtil::CreateDirectory(tmpdir));
   }
   return tmpdir;
 }
 
-#else  // OS_WINDOWS
+#else  // OS_WIN
 
+#ifndef MOZC_USE_PEPPER_FILE_IO
 // Get absolute path to this executable. Corresponds to argv[0] plus
 // directory information. E.g like "/spam/eggs/foo_unittest".
 string GetProgramPath() {
@@ -96,29 +102,37 @@ string GetProgramPath() {
   char cwd_buf[PATH_MAX+1];
   CHECK_GT(getcwd(cwd_buf, PATH_MAX), 0);
   cwd_buf[PATH_MAX] = '\0';  // make sure it's terminated
-  return Util::JoinPath(cwd_buf, program_invocation_name);
+  return FileUtil::JoinPath(cwd_buf, program_invocation_name);
 }
+#endif  // MOZC_USE_PEPPER_FILE_IO
 
 string GetTestSrcdir() {
-  // Use runfiles directory from program filename
-  const string srcdir = MOZC_DATA_DIR;
+  const string srcdir(kMozcDataDir);
 
+#ifndef MOZC_USE_PEPPER_FILE_IO
+  // TestSrcdir is not supported in NaCl.
+  // TODO(horo): Consider how to implement TestSrcdir in NaCl.
   // FIXME(komatsu): We should implement "genrule" and "exports_files"
   // in build.py to install the data files into srcdir.
   CHECK_EQ(access(srcdir.c_str(), R_OK|X_OK), 0)
-    << "Access failure: " << srcdir;
+      << "Access failure: " << srcdir;
+#endif  // MOZC_USE_PEPPER_FILE_IO
   return srcdir;
 }
 
+#ifndef MOZC_USE_PEPPER_FILE_IO
 string GetTestTmpdir() {
   const string tmpdir = GetProgramPath() + ".tmp";
 
+  // GetTestTmpdir is not supported in NaCl.
+  // TODO(horo): Consider how to implement TestTmpdir in NaCl.
   if (access(tmpdir.c_str(), R_OK|X_OK) != 0) {
-    CHECK(Util::CreateDirectory(tmpdir));
+    CHECK(FileUtil::CreateDirectory(tmpdir));
   }
   return tmpdir;
 }
-#endif  // OS_WINDOWS
+#endif  // MOZC_USE_PEPPER_FILE_IO
+#endif  // OS_WIN
 
 }  // namespace
 
@@ -127,7 +141,11 @@ void InitTestFlags() {
     FLAGS_test_srcdir = GetTestSrcdir();
   }
   if (FLAGS_test_tmpdir.empty()) {
+#ifndef MOZC_USE_PEPPER_FILE_IO
     FLAGS_test_tmpdir = GetTestTmpdir();
+#else  // MOZC_USE_PEPPER_FILE_IO
+    FLAGS_test_tmpdir = "/";
+#endif  // MOZC_USE_PEPPER_FILE_IO
   }
 }
 
