@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,9 +29,12 @@
 
 #include "unix/ibus/key_translator.h"
 
-#include <ibus.h>
+#include <map>
+#include <string>
 
 #include "base/logging.h"
+#include "base/port.h"
+#include "unix/ibus/ibus_extra_keysyms.h"
 
 namespace {
 
@@ -52,7 +55,11 @@ const struct SpecialKeyMap {
   {IBUS_Henkan, mozc::commands::KeyEvent::HENKAN},
   {IBUS_Muhenkan, mozc::commands::KeyEvent::MUHENKAN},
   {IBUS_Hiragana, mozc::commands::KeyEvent::KANA},
-  {IBUS_Katakana, mozc::commands::KeyEvent::KANA},
+  {IBUS_Hiragana_Katakana, mozc::commands::KeyEvent::KANA},
+  {IBUS_Katakana, mozc::commands::KeyEvent::KATAKANA},
+  {IBUS_Zenkaku, mozc::commands::KeyEvent::HANKAKU},
+  {IBUS_Hankaku, mozc::commands::KeyEvent::HANKAKU},
+  {IBUS_Zenkaku_Hankaku, mozc::commands::KeyEvent::HANKAKU},
   {IBUS_Eisu_toggle, mozc::commands::KeyEvent::EISU},
   {IBUS_Home, mozc::commands::KeyEvent::HOME},
   {IBUS_End, mozc::commands::KeyEvent::END},
@@ -84,6 +91,19 @@ const struct SpecialKeyMap {
   {IBUS_Page_Up, mozc::commands::KeyEvent::PAGE_UP},
   {IBUS_Page_Down, mozc::commands::KeyEvent::PAGE_DOWN},
 
+#ifdef OS_CHROMEOS
+  {IBUS_Back, mozc::commands::KeyEvent::F1},
+  {IBUS_Forward, mozc::commands::KeyEvent::F2},
+  {IBUS_Reload, mozc::commands::KeyEvent::F3},
+  {IBUS_LaunchB, mozc::commands::KeyEvent::F4},
+  {IBUS_LaunchA, mozc::commands::KeyEvent::F5},
+  {IBUS_MonBrightnessDown, mozc::commands::KeyEvent::F6},
+  {IBUS_MonBrightnessUp, mozc::commands::KeyEvent::F7},
+  {IBUS_AudioMute, mozc::commands::KeyEvent::F8},
+  {IBUS_AudioLowerVolume, mozc::commands::KeyEvent::F9},
+  {IBUS_AudioRaiseVolume, mozc::commands::KeyEvent::F10},
+#endif  // OS_CHROMEOS
+
   // Keypad (10-key).
   {IBUS_KP_0, mozc::commands::KeyEvent::NUMPAD0},
   {IBUS_KP_1, mozc::commands::KeyEvent::NUMPAD1},
@@ -115,6 +135,11 @@ const struct SpecialKeyMap {
   {IBUS_KP_End, mozc::commands::KeyEvent::END},
   {IBUS_KP_Delete, mozc::commands::KeyEvent::DEL},
   {IBUS_KP_Insert, mozc::commands::KeyEvent::INSERT},
+  {IBUS_Caps_Lock, mozc::commands::KeyEvent::CAPS_LOCK},
+
+  // This key event is for hangul IME. Do not handle this key event on Japanese
+  // IME.
+  {IBUS_Hangul_Hanja, mozc::commands::KeyEvent::HANJA},
 
   // Shift+TAB.
   {IBUS_ISO_Left_Tab, mozc::commands::KeyEvent::TAB},
@@ -127,12 +152,13 @@ const struct ModifierKeyMap {
   guint from;
   mozc::commands::KeyEvent::ModifierKey to;
 } modifier_key_map[] = {
-  {IBUS_Shift_L, mozc::commands::KeyEvent::SHIFT},
-  {IBUS_Shift_R, mozc::commands::KeyEvent::SHIFT},
-  {IBUS_Control_L, mozc::commands::KeyEvent::CTRL},
-  {IBUS_Control_R, mozc::commands::KeyEvent::CTRL},
-  {IBUS_Alt_L, mozc::commands::KeyEvent::ALT},
-  {IBUS_Alt_R, mozc::commands::KeyEvent::ALT},
+  {IBUS_Shift_L, mozc::commands::KeyEvent::LEFT_SHIFT},
+  {IBUS_Shift_R, mozc::commands::KeyEvent::RIGHT_SHIFT},
+  {IBUS_Control_L, mozc::commands::KeyEvent::LEFT_CTRL},
+  {IBUS_Control_R, mozc::commands::KeyEvent::RIGHT_CTRL},
+  {IBUS_Alt_L, mozc::commands::KeyEvent::LEFT_ALT},
+  {IBUS_Alt_R, mozc::commands::KeyEvent::RIGHT_ALT},
+  {IBUS_LOCK_MASK, mozc::commands::KeyEvent::CAPS},
 };
 
 const struct ModifierMaskMap {
@@ -144,7 +170,7 @@ const struct ModifierMaskMap {
   {IBUS_MOD1_MASK, mozc::commands::KeyEvent::ALT},
 };
 
-// TODO:Add kana_map_dv to support Dvoraklayout.
+// TODO(team): Add kana_map_dv to support Dvoraklayout.
 const struct KanaMap {
   guint code;
   const char *no_shift;
@@ -356,6 +382,7 @@ KeyTranslator::KeyTranslator() {
 KeyTranslator::~KeyTranslator() {
 }
 
+// TODO(nona): Fix 'Shift-0' behavior b/4338394
 bool KeyTranslator::Translate(guint keyval,
                               guint keycode,
                               guint modifiers,
@@ -365,38 +392,49 @@ bool KeyTranslator::Translate(guint keyval,
   DCHECK(out_event) << "out_event is NULL";
   out_event->Clear();
 
+  // Due to historical reasons, many linux ditributions set Hiragana_Katakana
+  // key as Hiragana key (which is Katkana key with shift modifier). So, we
+  // translate Hiragana_Katanaka key as Hiragana key by mapping table, and
+  // Shift + Hiragana_Katakana key as Katakana key by functionally.
+  // TODO(nona): Fix process modifier to handle right shift
+  if (IsHiraganaKatakanaKeyWithShift(keyval, keycode, modifiers)) {
+    modifiers &= ~IBUS_SHIFT_MASK;
+    keyval = IBUS_Katakana;
+  }
   string kana_key_string;
   if ((method == config::Config::KANA) && IsKanaAvailable(
           keyval, keycode, modifiers, layout_is_jp, &kana_key_string)) {
     out_event->set_key_code(keyval);
     out_event->set_key_string(kana_key_string);
   } else if (IsAscii(keyval, keycode, modifiers)) {
+    if (IBUS_LOCK_MASK & modifiers) {
+      out_event->add_modifier_keys(commands::KeyEvent::CAPS);
+    }
     out_event->set_key_code(keyval);
   } else if (IsModifierKey(keyval, keycode, modifiers)) {
     ModifierKeyMap::const_iterator i = modifier_key_map_.find(keyval);
     DCHECK(i != modifier_key_map_.end());
-    out_event->add_modifier_keys((*i).second);
+    out_event->add_modifier_keys(i->second);
   } else if (IsSpecialKey(keyval, keycode, modifiers)) {
     SpecialKeyMap::const_iterator i = special_key_map_.find(keyval);
     DCHECK(i != special_key_map_.end());
-    out_event->set_special_key((*i).second);
+    out_event->set_special_key(i->second);
   } else {
     VLOG(1) << "Unknown keyval: " << keyval;
     return false;
   }
 
   for (ModifierKeyMap::const_iterator i = modifier_mask_map_.begin();
-       i != modifier_mask_map_.end();
-       ++i) {
+       i != modifier_mask_map_.end(); ++i) {
     // Do not set a SHIFT modifier when |keyval| is a printable key by following
     // the Mozc's rule.
-    if (((*i).second == commands::KeyEvent::SHIFT) &&
-        IsAscii(keyval, keycode, modifiers)) {
+    if ((i->second == commands::KeyEvent::SHIFT) &&
+        IsPrintable(keyval, keycode, modifiers)) {
       continue;
     }
 
-    if ((*i).first & modifiers) {
-      out_event->add_modifier_keys((*i).second);
+    if (i->first & modifiers) {
+      out_event->add_modifier_keys(i->second);
     }
   }
 
@@ -440,6 +478,12 @@ bool KeyTranslator::IsSpecialKey(guint keyval,
   return special_key_map_.find(keyval) != special_key_map_.end();
 }
 
+bool KeyTranslator::IsHiraganaKatakanaKeyWithShift(guint keyval,
+                                                   guint keycode,
+                                                   guint modifiers) {
+  return ((modifiers & IBUS_SHIFT_MASK) && (keyval == IBUS_Hiragana_Katakana));
+}
+
 bool KeyTranslator::IsKanaAvailable(guint keyval,
                                     guint keycode,
                                     guint modifiers,
@@ -473,9 +517,17 @@ bool KeyTranslator::IsKanaAvailable(guint keyval,
   return true;
 }
 
-bool KeyTranslator::IsAscii(guint keyval,
-                            guint keycode,
-                            guint modifiers) {
+// TODO(nona): resolve S-'0' problem (b/4338394).
+// TODO(nona): Current printable detection is weak. To enhance accuracy, use xkb
+// key map
+bool KeyTranslator::IsPrintable(guint keyval, guint keycode, guint modifiers) {
+  if ((modifiers & IBUS_CONTROL_MASK) || (modifiers & IBUS_MOD1_MASK)) {
+    return false;
+  }
+  return IsAscii(keyval, keycode, modifiers);
+}
+
+bool KeyTranslator::IsAscii(guint keyval, guint keycode, guint modifiers) {
   return (keyval > IBUS_space &&
           // Note: Space key (0x20) is a special key in Mozc.
           keyval <= IBUS_asciitilde);  // 0x7e.
